/*
 * Copyright (c) 2005 The University of Reading
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Reading, nor the names of the
 *    authors or contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.rdg.resc.jstyx.messages;

import uk.ac.rdg.resc.jstyx.StyxUtils;
import uk.ac.rdg.resc.jstyx.types.DirEntry;

/**
 * Message sent to change the attributes of a file
 * @todo Implement getFriendlyString()
 *
 * @author Jon Blower
 * $Revision: 1.4 $
 * $Date: 2005/03/15 09:01:48 $
 * $Log: TwstatMessage.java,v $
 * Revision 1.4  2005/03/15 09:01:48  jonblower
 * Message type now stored as short, not int
 *
 * Revision 1.3  2005/03/11 14:02:16  jonblower
 * Merged MINA-Test_20059309 into main line of development
 *
 * Revision 1.2.2.1  2005/03/10 11:50:59  jonblower
 * Changed to fit with MINA framework
 *
 * Revision 1.2  2005/02/24 07:44:44  jonblower
 * Added getFriendlyString()
 *
 * Revision 1.1.1.1  2005/02/16 18:58:30  jonblower
 * Initial import
 *
 */
public class TwstatMessage extends StyxMessage
{
    
    private short fid;  // Fid chosen by client to represent root of file tree
    private DirEntry dirEntry; // File attributes
    
    /** 
     * Creates a new TwstatMessage. This constructor will be called by the
     * MessageRecognizer.
     * @param length The total length of the message (including all header info)
     * @param type The type of the message (a number between 100 and 127)
     * @param tag The tag that identifies this message
     */
    public TwstatMessage(int length, int tag)
    {
        super(length, (short)24, tag);
        this.name = "Twstat";
    }
    
    public TwstatMessage(short fid, DirEntry dirEntry)
    {
        this(0, 0); // The length and tag will be added later
        this.fid = fid;
        this.dirEntry = dirEntry;
        // Set the length of the message
        this.length = StyxUtils.HEADER_LENGTH + 2 + this.dirEntry.getSize();
    }
        
    protected final void decodeBody(StyxBuffer buf)
    {
        this.fid = buf.getUShort();
        int sizeOfDirEntry = buf.getUShort(); // size of DirEntry, ignored
        this.dirEntry = buf.getDirEntry();
    }
    
    protected final void encodeBody(StyxBuffer buf)
    {
        buf.putUShort(this.fid).putDirEntry(this.dirEntry);
    }
    
    public short getFid()
    {
        return this.fid;
    }
    
    public DirEntry getDirEntry()
    {
        return this.dirEntry;
    }
    
    protected String getElements()
    {
        return ", " + this.fid + ", " + this.dirEntry;
    }
    
}
