#include <sys/types.h>
#ifndef MYSTYX_H
#define MYSTYX_H

#define	VERSION9P	"9P"

#define MAXREAD         119
#define MAXWRITE        111
#define	MAXFDATA	120
#define	MAXMSG		128	/* max header sans data */
#define NOTAG		0xFFFF	/* Dummy tag */


/* bits in Qid.type */
#define QTDIR		0x80		/* type bit for directories */
#define QTAPPEND	0x40		/* type bit for append only files */
#define QTEXCL		0x20		/* type bit for exclusive use files */
#define QTMOUNT		0x10		/* type bit for mounted channel */
#define QTAUTH		0x08		/* type bit for authentication file */
#define QTFILE		0x00		/* plain file */

#define	OREAD	0	/* open for read */
#define	OWRITE	1	/* write */
#define	ORDWR	2	/* read and write */
#define	OEXEC	3	/* execute, == read but check execute permission */
#define	OTRUNC	16	/* or'ed in (except for exec), truncate file first */
#define	OCEXEC	32	/* or'ed in, close on exec */
#define	ORCLOSE	64	/* or'ed in, remove on close */
#define OEXCL   0x1000	/* or'ed in, exclusive create */

/* bits in Dir.mode */
#define DMDIR		0x80000000	/* mode bit for directories */
#define DMAPPEND	0x40000000	/* mode bit for append only files */
#define DMEXCL		0x20000000	/* mode bit for exclusive use files */
#define DMMOUNT		0x10000000	/* mode bit for mounted channel */
#define DMREAD		0x4		/* mode bit for read permission */
#define DMWRITE		0x2		/* mode bit for write permission */
#define DMEXEC		0x1		/* mode bit for execute permission */


#define	ERRLEN		64
#define	DIRLEN		116
#define	NAMELEN		28



typedef	unsigned char	uchar;
typedef	long long	vlong;
typedef	unsigned long long	uvlong;

# ifndef __int8_t_defined
#  define __int8_t_defined
typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;
typedef unsigned int u_int32_t;
typedef unsigned long long u_int64_t;
#endif
typedef unsigned int            u32int;

typedef struct Qid
{
	ulong	path;
	ulong	vers;
} Qid;

typedef struct Dir
{
	char	name[NAMELEN];
	char	uid[NAMELEN];
	char	gid[NAMELEN];
	Qid	qid;
	ulong	mode;
	long	atime;
	long	mtime;
        u_int64_t length;   
	short	type;
	short	dev;
} Dir;

typedef	struct	Fcall	Fcall;
struct	Fcall
{
	char	type;
	short	fid;
	ushort	tag;
	ushort	oldtag;		/* T-Flush */
	Qid	qid;		/* R-Attach, R-Walk, R-Open, R-Create */
	char	uname[NAMELEN];	/* T-Attach */
	char	aname[NAMELEN];	/* T-Attach */
	char	ename[ERRLEN];	/* R-Error */
	long	perm;		/* T-Create */ 
	short	newfid;		/* T-Clone */
	char	name[NAMELEN];	/* T-Walk, T-Create */
	char	mode;		/* T-Create, T-Open */
	long	offset;		/* T-Read, T-Write */
	long	count;		/* T-Read, T-Write, R-Read */
	char	*data;   	/* T-Write, R-Read */
	char	stat[DIRLEN];	/* T-Wstat, R-Stat */
};

enum {
        Tnop,           //  0
        Rnop,           //  1
        Terror,         //  2, illegal
        Rerror,         //  3
        Tflush,         //  4
        Rflush,         //  5
        Tclone,         //  6
        Rclone,         //  7
        Twalk,          //  8
        Rwalk,          //  9
        Topen,          // 10
        Ropen,          // 11
        Tcreate,        // 12
        Rcreate,        // 13
        Tread,          // 14
        Rread,          // 15
        Twrite,         // 16
        Rwrite,         // 17
        Tclunk,         // 18
        Rclunk,         // 19
        Tremove,        // 20
        Rremove,        // 21
        Tstat,          // 22
        Rstat,          // 23
        Twstat,         // 24
        Rwstat,         // 25
        Tsession,       // 26 -- unimplemented
        Rsession,       // 27 -- unimplemented
        Tattach,        // 28
        Rattach,        // 29
	Tmax
};

static char msglen[Tmax] =
{
	[Tnop]	   3,
	[Terror]   0,
	[Rerror]  67,
	[Tflush]   5,
	[Rflush]   3,
	[Tclone]   7,
	[Rclone]   5,
	[Twalk]	  33,
	[Rwalk]	  13,
	[Topen]	   6,
	[Ropen]	  13,
	[Tcreate] 38,
	[Rcreate] 13,
	[Tread]	  15,
	[Rread]	   8,	/*header only; excludes data */
	[Twrite]  16,   /* header only; excludes data */
	[Rwrite]   7,
	[Tclunk]   5,
	[Rclunk]   5,
	[Tremove]  5,
	[Rremove]  5,
	[Tstat]	   5,
	[Rstat]	 121,
	[Twstat] 121,
	[Rwstat]   5,
	[Tattach] 5+2*NAMELEN,
	[Rattach]  13
};


int	convM2S(uchar*, Fcall*, int);
int	convS2M(Fcall*, uchar*);

int	convM2D(char*, Dir*);
int	convD2M(Dir*, char*, int );
#endif
