#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termio.h>
#include <termios.h>
#include <semaphore.h>

#include "slib.h"

extern int styx_rd, styx_wr;

#define MAX_TAGS 0xFFFF

#define SND_DELAY 1000

static unsigned short currentTag = 0;

sem_t sem_ser;

int serialfd; // gloabl fd for the serial port

static unsigned short nextTag() {
  unsigned short ret = currentTag;
  currentTag = (currentTag + 1) & MAX_TAGS;
  return ret;
}

static int open_serial_port()
{
   struct termios options;
   int fd;
      
   //   fd = open("/dev/ttyS0",O_RDWR|O_NOCTTY|O_NDELAY);
   fd = open("/dev/ttyS0",O_RDWR|O_NOCTTY|O_SYNC);
   if (fd == -1) 
   {	 
    	 printf("Could Not Open tty\n");
	 perror("open error");
	 return (-1);
   }
   tcgetattr(fd,&options);
   //  cfsetispeed(&options,B38400);
   // cfsetospeed(&options,B38400);
   cfsetispeed(&options,B115200);
   cfsetospeed(&options,B115200);
   options.c_cflag &= ~PARENB;
   options.c_cflag &= ~CSTOPB;
   options.c_cflag &= ~CSIZE;
   options.c_cflag |= CS8;
   options.c_cflag &= ~CRTSCTS;
   options.c_lflag &= ~(ICANON|ECHO|ECHOE|ISIG);
   options.c_cflag |= (CLOCAL|CREAD);
   //   options.c_cc[VMIN] = 0;
   //   options.c_cc[VTIME] = 0;
   options.c_cc[VMIN] = 1;
   options.c_cc[VTIME] = 100;
   tcsetattr(fd,TCSANOW,&options);

   sem_init(&sem_ser, 0, 1);
   return (fd);
}


// this code needs to be generalized so that it works
// both ways -- server and client

static int snd(Fcall * out) {
  uchar buffer[MAXMSG];
  int len;
  int n;
  
  if (sem_wait(&sem_ser) < 0) {
    printf("error waiting for sem\n");
    return(-1);
  }
    

  len = convS2M(out, buffer);
  printf("msg type %d, len = %d\n", out->type, len);
  if (len == -1)
    return STYX_ERROR;
  
  // need to do some error checking here

  for(n = 0; n<len ;n++) {
    uchar uc;

    uc = buffer[n] & 0xF;
    
    uc = uc + '0';
    if (write(serialfd, &uc, 1) < 0)
      return STYX_ERROR;
    tcflush(serialfd, TCIFLUSH);
    ///    printf("put :: %d, ", uc);
    usleep(SND_DELAY);   // sleep for a millisec
    
    uc = buffer[n] >> 4;

    uc = uc + '0';
    if (write(serialfd, &uc, 1) < 0)
      return STYX_ERROR;
    tcflush(serialfd, TCIFLUSH);
    usleep(SND_DELAY);   // sleep for a millisec
    ///    printf(" %d\n", uc);

  }


  if (out->type == Twrite || out->type == Rread)
    for(n = 0; n<out->count ;n++) {
      uchar uc;
      
      uc = out->data[n] & 0xF;
      uc = uc + '0';      

      if (write(serialfd, &uc, 1) < 0)
	return STYX_ERROR;
      tcflush(serialfd, TCIFLUSH);
      usleep(SND_DELAY);   // sleep for a millisec
      printf("put :: %d, ", uc);
      
      uc = out->data[n] >> 4;
      uc = uc + '0';
      if (write(serialfd, &uc, 1) < 0)
	return STYX_ERROR;
      tcflush(serialfd, TCIFLUSH);
      usleep(SND_DELAY);   // sleep for a millisec
      printf(" %d\n", uc);
      
    }

  return STYX_OK;
}


static int rread(char *buf, int len)
{
  int ret;
  int i = 0;
  int rlen;
  uchar ch1, ch2;

  while (i != len)
    {
      if ((rlen = read(serialfd, &ch1, 1)) != 1)
	{
	  printf("error in serial recv: rlen = %d\n",rlen);
	  return STYX_ERROR;
	}
      ///      printf("get :: %d\n", ch1);
      if ((rlen = read(serialfd, &ch2, 1)) != 1)
	{
	  printf("error in serial recv: rlen = %d\n",rlen);
	  return STYX_ERROR;
	}
      ///      printf("get :: %d\n", ch2);
      
      ch1 = ch1 - '0'; ch2 = ch2 - '0';

      buf[i] = ch1 + (ch2 << 4);

      i++;
    }
  return len;
}


static int rcv(Fcall *in) {
  char buffer[MAXMSG];
  int type;
  int len;
  int rlen;
  int blen;
  int cnt;

  if ((rlen = rread(buffer, 1)) != 1)
    {
      fprintf(stderr, "recv error: expected 1, got %d\n",rlen);
      return STYX_ERROR;
    }

  type = buffer[0];
  printf("rcv: got message of type %d\n", type);

  // read message header

  len = msglen[buffer[0]];

  if ((rlen = rread(&buffer[1], len - 1)) != len - 1)
    {
      fprintf(stderr, "recv error: expected %d, got %d\n",len - 1, rlen);
      return STYX_ERROR;
    }

  // now tread Twrite, Rread specially

  blen = 0;

  if (type == Twrite)
    {
      blen = buffer[13] | (buffer[14] << 8);
      cnt = blen > MAXWRITE ? MAXWRITE : blen;
    }
  if (type == Rread)
    {
      blen = buffer[5] | (buffer[6] << 8);
      cnt = blen > MAXREAD ? MAXREAD : blen;
    }
  if (blen)
    {

      char c;
      if ((rlen = rread(in->data, cnt)) != cnt)
	{
	  fprintf(stderr, "recv error: expected %d, got %d\n",cnt, rlen);
	  return STYX_ERROR;
	} 

      /* dump extra characters */

      for (cnt = blen  - cnt; cnt; cnt--)
	if (read(styx_rd, &c, 1) != 1)
	  {
	    printf("trouble read/write more than limit!\n");
	    return STYX_ERROR;
	  }
    }
  convM2S(buffer, in, blen + len);
  if (sem_post(&sem_ser) < 0) {
    printf("error releasing sem\n");
    return(-1);
  }  
  return STYX_OK;
}

int rpc(Fcall * in, Fcall * out) {
  int v = snd(out);
  if (v == STYX_ERROR)
    return v;
  v = rcv(in);
  if (v == STYX_ERROR)
    return v;
  if ((int) in->type == ((int) out->type+1))
    return STYX_OK;
  else
    return STYX_ERROR;
}

int styx_attach(int fid, Qid *rootqid) {
  Fcall in, out;
  bzero(&out, sizeof(out));
  out.type = Tattach;
  out.fid =  fid;
  out.tag = nextTag();
  strcpy(out.uname,"");
  strcpy(out.aname,"");

  if (rpc(&in, &out) == STYX_ERROR) return STYX_ERROR;
  if (rootqid) memcpy((void *)rootqid, &(in.qid), sizeof(Qid));
  return STYX_OK;
}

int styx_walk(int fid, char* name, Qid *wqid) {
  int i;
  Fcall in, out;
  out.type = Twalk;
  out.tag = nextTag();
  out.fid = fid;
  strcpy(out.name, name);

  printf("  walking fid <%d> to %s\n", fid, name);
  if (rpc(&in, &out) == STYX_ERROR) return STYX_ERROR;
  if (wqid) memcpy((void *)wqid, (void *) &in.qid, sizeof(Qid));
  return STYX_OK;
}

int styx_read(int fid, vlong offset, u32int count, char* buffer) {
  Fcall in, out;
  int i = 0;

  out.type = Tread;
  out.tag = nextTag();
  out.fid = fid;

  while (i < count)
    {
      out.offset = offset + i;
      out.count = count - i > MAXREAD ? MAXREAD : count - i;
      in.data = &buffer[i];
      if (rpc(&in, &out) == STYX_ERROR) return STYX_ERROR;
      //      bcopy(in.data, &buffer[i], in.count);
      i += in.count;
      if (in.count == 0)
	break;
    }
  return i;
}


int styx_write(int fid, vlong offset, u32int count, char* buffer) {
  Fcall in, out;
  int i = 0;

  out.type = Twrite;
  out.fid = fid;
  out.tag = nextTag();

  /* keep writing till we are have written 'em all  */

  while (i < count)
    {
      out.offset = offset + i;
      out.count = count - i > MAXWRITE ? MAXWRITE : count - i;
      out.data = &buffer[i];
      //      memcpy(&out.data, &buffer[i], out.count);
      if (rpc(&in, &out) == STYX_ERROR) return STYX_ERROR;
      i += in.count;
      if (in.count == 0)
	break;
    }
  return i;
}

int styx_clunk(int fid) {
  Fcall in, out;
  out.type = Tclunk;
  out.tag = nextTag();
  out.fid = fid;
  if (rpc(&in, &out) == STYX_ERROR) {
    return STYX_ERROR;
  }
  return STYX_OK;
}

int styx_open(int fid, int mode) {

  Fcall in, out;
  out.type = Topen;
  out.tag = nextTag();
  out.fid = fid;
  out.mode = mode;
  if (rpc(&in, &out) == STYX_ERROR) {
    return STYX_ERROR;
  }
  return STYX_OK;
}

int styx_stat(int fid, Dir* stat) {
  Fcall in, out;
  out.type = Tstat;
  out.tag = nextTag();
  out.fid = fid;

  if (rpc(&in, &out) == STYX_ERROR) {
    return STYX_ERROR;
  }

  convM2D(in.stat, stat); 
  return STYX_OK;
}

int styx_clone(int fid, int nfid) {

  Fcall in, out;
  out.type = Tclone;
  out.tag = nextTag();
  out.fid = fid;
  out.newfid = nfid;
  if (rpc(&in, &out) == STYX_ERROR) {
    return STYX_ERROR;
  }
  return STYX_OK;
}


int styx_connect_serial() {
  serialfd = open_serial_port();
  return STYX_OK;
} 
