/*
 * target/target.c: implements the interface to the different arch targets
 *
 * Copyright (C) 2003, Rongkai Zhan <zhanrk@163.com>, All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: arm7target.c,v 1.1.1.1 2007/06/05 20:06:30 bpisupat Exp $ */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "jtag.h"
#include "arm7target.h"
#include "arm7tdmi.h"

/* the unified interface of different target implementation */
struct target_struct *target = NULL;
struct target_operation *t_op = NULL;

core_register_t * target_register_index(const char *regname)
{
	core_register_t *reg = NULL;
	int i = 0;
	int found = 0;

	if (!target || !target->regs)
		return NULL;
	
	while (1) {
		reg = target->regs + i;
		if (reg->name == NULL)
			break;
		if (strcasecmp(regname, reg->name) == 0) {
			found = 1;
			break;
		}
		i ++;
	}
	if (found == 0)
		reg = NULL;
	return reg;
} /* end of target_register_index(...) */

ice_register_t * target_ice_register_index(const char *regname)
{
	ice_register_t *reg;
	int i = 0;
	int found = 0;
	
	if (!target || !target->ice_regs)
		return NULL;

	while (1) {
		reg = target->ice_regs + i;
		if (reg->name == NULL)
			break;
		if (strcasecmp(reg->name, regname) == 0) {
			found = 1;
			break;
		}
		i ++;
	}
	if (found == 0)
		reg = NULL;
	return reg;
} /* end of target_ice_register_index(...) */

int target_setup(int type)
{
	int retval = 0;
	
	if (type == TARGET_TYPE_ARM7TDMI)
		retval = arm7tdmi_setup();
	 else {
		printf("Error: %s: invalid target type.\n", __FUNCTION__);
		retval = -1;
	}
	
	return retval;
} /* end of target_setup() */

void arm7_initialize() 
{
  u32 idcode;
  u32 memval = 0xabcd;
  u32 readmemval[2];

  int ret = 0;
  /* Initialize jtag */
  if (jtag_reset()) {
    printf("Error: %s: failure to initialize jtag ...\n");
    return -1;
  }

  arm7tdmi_setup();
  jtag_idcode(&idcode);
  printf("ARM7 device idcode = %x\n",idcode);
  ret = arm7tdmi_halt();
  if (ret) {
    printf("halt error\n"); 
    return -1;
  }

  printf("halted successfully\n");
}

int ishalted() {
  u32 status;
  int retval;

  retval = arm7tdmi_ice_read(ARM7TDMI_ICE_DBGSTAT, &status);
  if (retval) {
    printf("error in reading DBGSTAT reg\n");
    return -1;
  }
    
    if (status & 0x01) {
      /* success */

    target->status = TARGET_STATUS_HALTED;
    target->halt_reason = TARGET_HALTED_BY_DBGRQ;
    
    printf("The target is halted in ");
    if (status & 0x10) {
      printf("THUMB mode.\n");
      target->mode = TARGET_MODE_THUMB;
    } else {
      printf("ARM mode.\n");
      target->mode = TARGET_MODE_ARM;
    }
    
      return 1;
    } else
      return 0;
}
    
