#include <unistd.h>
#include <sys/time.h>
#include "system.h"
#include "MSP/Basic_Types.h"
#include "altera_avalon_pio_regs.h"
#include "MSP/HIL.h"


//#define MICROSEC_ITER 3
#define MICROSEC_ITER 10

#define MILLISEC_ITER 3200

int piodata;
struct timeval starttime;
int endtime;

#define SetTest() HIL_TST(1)
#define ClrTest() HIL_TST(0)
#define SetTCK() HIL_TCK(1)
#define ClrTCK() HIL_TCK(0)
#define SetTMS() HIL_TMS(1)
#define ClrTMS() HIL_TMS(0)
#define SetTDI() HIL_TDI(1)
#define ClrTDI() HIL_TDI(0)
#define SetTCLK() HIL_TCLK(1)
#define ClrTCLK() HIL_TCLK(0)
#define SetReset() HIL_RST(1) 

#define Delay(x) HIL_DelayMSec(x)

void ResetTAP(void);

/*----------------------------------------------------------------------------
   Initialization of the Target Board (switch voltages on, preset JTAG pins)	
*/
STATUS_T WINAPI HIL_Initialize(CHAR* port)
{
 int i;
 struct timeval starttime;
  /*
 RESET 2
 TEST 3
 TDI 4  <-- only input
 TMS 5
 TCLK 4   // same as TDI
 TCK 6
 TDO 7 */

  //set direction for pio pins: outputs are 1

   i = (1 << RESET) + (1 << TEST) + (1 << TMS) + (1 << TCK) + (1 << TDI) ;
  IOWR_ALTERA_AVALON_PIO_DIRECTION(JTAGPIO_BASE, i);
  //set all signals to be low
   ResetSignals(); 

  /* Power is set externally (in hardware); so no need to set it inside */
  //  SetPower();

    SetReset();

    ResetTAP();
    
   starttime.tv_sec = 0;
   starttime.tv_usec = 0;
   settimeofday(&starttime, NULL);

    return STATUS_OK;
  
}


void ResetSignals()
{
   IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, 0);
  piodata = 0;
}

STATUS_T WINAPI HIL_Open(void)
{
  return STATUS_OK;
}

STATUS_T WINAPI HIL_Connect(void)
{
  return STATUS_OK;
}

STATUS_T WINAPI HIL_Release(void)
{
  return STATUS_OK;
}

STATUS_T WINAPI HIL_Close(LONG vccOff)
{
  return STATUS_OK;
}

LONG WINAPI HIL_JTAG_IR(LONG instruction)
{
  word j;

  // JTAG FSM state = Run-Test/Idle
  SetTMS();
  /*  ClrTCK();
  microDelay(3);
  SetTCK();
  */

  ClrTCK();

  
  //for (j = 5; j > 0; j--);
  microDelay(1);

  SetTCK();
		

  microDelay(4);
  // JTAG FSM state = Select DR-Scan
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);
  // JTAG FSM state = Select IR-Scan
  ClrTMS();
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);
  // JTAG FSM state = Capture-IR
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);
  // JTAG FSM state = Shift-IR, Shift in TDI (8-bit)
  return(Shift(F_BYTE, instruction));
  // JTAG FSM state = Run-Test/Idle
}

LONG WINAPI HIL_TEST_VPP(LONG mode)
{
  /* if mode is 'FALSE' (= 0???) then JTAG_IR needs to start shifting in thru TDO
     which is something we don't do. So go into infinite loop */
  if (!mode)
    while(1);
  return 1; //return value is 'TRUE' (which means 1 would hopefully work)
}

LONG WINAPI HIL_JTAG_DR(LONG data, LONG bits)
{
  // JTAG FSM state = Run-Test/Idle
  SetTMS();
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);  
  // JTAG FSM state = Select DR-Scan
  ClrTMS();
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);
  // JTAG FSM state = Capture-DR
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);
  // JTAG FSM state = Shift-DR, Shift in TDI (16-bit)
  if (bits == 16)
    return(Shift(F_WORD, data));
  else
    return(Shift(F_BYTE, data));
  // JTAG FSM state = Run-Test/Idle  
}

STATUS_T WINAPI HIL_VCC(LONG voltage)
{
  return STATUS_OK;
}

void WINAPI HIL_TST(LONG state)
{
  if (state) {
  piodata |= (1 << TEST);
  }
  else {
  piodata &= ~(1 << TEST);
  }
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
}

void WINAPI HIL_TCK(LONG state)
{
  volatile int i;

  switch (state) {
  case 1:
    piodata |= (1 << TCK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    break;
  case 0:
    piodata &= ~(1 << TCK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    break;
  case POS_EDGE:
    piodata &= ~(1 << TCK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    for(i=0;i<1000;i++);
    piodata |= (1 << TCK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    break;
  case NEG_EDGE:
    piodata |= (1 << TCK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    for(i=0;i<1000;i++);
    piodata &= ~(1 << TCK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
}

void WINAPI HIL_TMS(LONG state)
{
  if (state) {
  piodata |= (1 << TMS);
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
  else {
  piodata &= ~(1 << TMS);
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
}

void WINAPI HIL_TDI(LONG state)
{
  if (state) {
  piodata |= (1 << TDI);
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
  else {
  piodata &= ~(1 << TDI);
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
}

void WINAPI HIL_TCLK(LONG state)
{
  volatile int i;

  switch (state) {
  case 1:
    piodata |= (1 << TCLK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    break;
  case 0:
    piodata &= ~(1 << TCLK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    break;
  case POS_EDGE:
    piodata &= ~(1 << TCLK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    for(i=0;i<1000;i++);
    piodata |= (1 << TCLK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    break;
  case NEG_EDGE:
    piodata |= (1 << TCLK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
    for(i=0;i<1000;i++);
    piodata &= ~(1 << TCLK);
    IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }


}

void WINAPI HIL_RST(LONG state)
{
  if (state) {
  piodata |= (1 << RESET);
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
  else {
  piodata &= ~(1 << RESET);
  IOWR_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE, piodata);
  }
}

STATUS_T WINAPI HIL_VPP(LONG voltage)
{
  return STATUS_OK;
}


/* this returns a zero value when TDO is 0 and non-zero value
   when tdo is non-zero */

int ScanTDO()
{
  volatile int rv;
  
  rv = IORD_ALTERA_AVALON_PIO_DATA(JTAGPIO_BASE);
  return ( rv & (1 << TDO));
}
	
/*----------------------------------------------------------------------------
   Release Target Board (switch voltages off, JTAG pins are HI-Z)	
*/

void ReleaseTarget(void)
{}



//----------------------------------------------------------------------------
/*  Shift a value into TDI (MSB first) and simultaneously shift out a value
    from TDO (MSB first).
    Note:      When defining SPI_MODE the embedded SPI is used to speed up by 2.
    Arguments: word Format (number of bits shifted, 8 (F_BYTE) or 16 (F_WORD))
               word Data (data to be shifted into TDI)  
    Result:    word (scanned TDO value)
*/
word Shift(word Format, word Data)
{
	word tclk = StoreTCLK();		// Store TCLK state;
	word TDOword = 0x0000;			// Initialize shifted-in word
	word MSB = 0x0000;
	word i;
	word j;
	word clkcount;
	word tdobit;

       // Shift via port pins, no coding necessary
	
	(Format == F_WORD) ? (MSB = 0x8000) : (MSB = 0x80);
	for (i = Format; i > 0; i--)
	{
		((Data & MSB) == 0) ? ClrTDI() : SetTDI();
		Data <<= 1;
		if (i == 1)			// Last bit requires TMS=1
		SetTMS();
		ClrTCK();

		//		intmicroDelay(3);

		for (j = 5; j > 0; j--);

		SetTCK();
		
		/* need to make the the effective freq of this clock
		   to be equal to 8 mhz:
		   8mhz == 125 nanosecs */
		
		
		//	for (j = 2; j > 0; j--);
		// intmicroDelay(4);

		TDOword <<= 1;			// TDO could be any port pin
		tdobit = ScanTDO();
		if (tdobit != 0)
			TDOword++;	
	}


	// common exit
	RestoreTCLK(tclk);  			// restore TCLK state
		PrepTCLK();				// Set JTAG FSM back into Run-Test/Idle
	return(TDOword);
}

//----------------------------------------------------------------------------
/* This function sets the target JTAG state machine (JTAG FSM) back into the 
   Run-Test/Idle state after a shift access.
*/
void PrepTCLK(void)
{
  // JTAG FSM = Exit-DR
  ClrTCK();
  microDelay(3);
  SetTCK();
  microDelay(4);
  // JTAG FSM = Update-DR
  ClrTMS();
  ClrTCK();
  microDelay(3);  
  SetTCK();
  microDelay(4);  

  // JTAG FSM = Run-Test/Idle
}



void WINAPI HIL_DelayMSec(LONG mSeconds)
{
  usleep(mSeconds*1000);
}

void WINAPI HIL_StartTimer(void)
{
  return;
  
}

LONG WINAPI HIL_ReadTimer(void)
{
  struct timeval endtime;
  LONG returntime;
  
  if (gettimeofday(&endtime, NULL) < 0)
    printf("WARNING: unable to get endtime: gettimeofday failed\n");
  returntime = endtime.tv_sec *1000 + endtime.tv_usec/1000;
  if (returntime<0) {
    printf("WARNING: returntime < 0!\n");   
  }
 // printf("end sec = %lu, end usec = %lu\n", endtime.tv_sec, endtime.tv_usec);
    
  return returntime;  
}

void WINAPI HIL_StopTimer(void)
{}



void microDelay(int microsec)			// Inner loop generates 1 micro
{
	volatile int i, j;
	for (i = microsec; i > 0; i--)
		for (j = 1; j < MICROSEC_ITER; j++) ;
}  


void ResetTAP(void)
{
  int i;

  //  At time = 0 set TDI, TMS ank TCK to be high
  SetTMS();
  SetTCK();
  SetTDI();

  //9 micros after that, set TEST to be high
  microDelay(10);
  SetTest();

  //9 micros after that, set TCK to be low
  microDelay(10);
  ClrTCK();

  //9 micros after that, set TMS to be low
  microDelay(10);
  ClrTMS();

  //62 millis after that set TMS to be high
  //  Delay(90);
  Delay(62);
  SetTMS();

  //62 millis after that set TMS to be low
  Delay(62);
  ClrTMS();

  //62 millis after that set TMS to be high
  Delay(62);
  SetTMS();
  Delay(62);

  for (i = 6; i > 0; i--)
    {
      microDelay(4);
      ClrTCK();
      microDelay(4);
      SetTCK();
    }
  // JTAG FSM is now in Test-Logic-Reset  
  microDelay(4);
  ClrTCK();
  ClrTMS();
  microDelay(4);
  SetTCK();
  microDelay(4);

}

