/*========================================================================*\
|                                                                          |
| MSP430.h                                                                 |
|                                                                          |
| This file contains the DLL function headers and definitions              |
|--------------------------------------------------------------------------|
| Project:              MSP430 JTAG interfcae                              |
| Developed using:      MS Visual C++ 5.0                                  |
|--------------------------------------------------------------------------|
| Version:              1.00                                               |
| Initial Version:      20 / 07 / 02                                       |
| Last Change:                                                             |
|--------------------------------------------------------------------------|
| Version history:                                                         |
| Version: 1.1.5                                                           |
| 01/21/2004 UPSF Added ramStart; ram2Start; ram2End to union DEVICE_T     |
| 01/21/2004 UPSF removed device enumeration tabel                         |
| 01/21/2004 UPSF added support for MarginalRead of Flash (autom. devices) |
|                                                                          |
|--------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments                                       |
\*========================================================================*/

/*------------------------------------------------------------------------*\
| Remarks:                                                                 |
|                                                                          |
\*------------------------------------------------------------------------*/

#ifndef _MSP430_H_
#define _MSP430_H_

// #includes. -----------------------------------------------------------------

#if defined(_MSC_VER)
// include this part if it is compiled with MS C Compiler
#include <windows.h>
#else
// All other compilers may have to define do some special defines in Basic_Types.h
#include <stddef.h>
#include "MSP/Basic_Types.h"
#endif


// #defines. ------------------------------------------------------------------

#ifndef _BASIC_TYPES_H_

typedef unsigned char BYTE;
typedef unsigned short WORD;

typedef WORD word;
typedef LONG STATUS_T;

// Status codes.
enum {
    STATUS_ERROR = -1,
    STATUS_OK,
};

#define _BASIC_TYPES_H_
#endif



union DEVICE_T {    // Device information structure.
    CHAR buffer[80];
    struct {
        WORD  endian;        // The value 0xaa55.
        WORD  id;            // Identification number.
        BYTE  string[32];    // Identification string.
        WORD  mainStart;     // MAIN MEMORY (FLASH) starting address.
        WORD  infoStart;     // INFORMATION MEMORY (FLASH) starting address.
        WORD  ramEnd;        // RAM ending address.
        WORD  nBreakpoints;  // Number of breakpoints.
        WORD  emulation;     // Emulation level.
        WORD  clockControl;  // Clock control level.
        WORD  lcdStart;      // LCD starting address.
        WORD  lcdEnd;        // LCD ending address.
        WORD  vccMinOp;      // Vcc minimum during operation [mVolts].
        WORD  vccMaxOp;      // Vcc maximum during operation [mVolts].
        WORD  hasTestVpp;    // Device has TEST/VPP.
        WORD  ramStart;      // RAM starting address.
        WORD  ram2Start;     // RAM2 starting address.
        WORD  ram2End;       // RAM2 ending address.
        WORD  infoEnd;       // INFO ending address.
        ULONG mainEnd;       // MAIN ending address.
    };
};

// Devices. One of the following enumerations is returned in device.id
//  -> removed 
#define DEVICE_UNKNOWN   0


enum
{
    WRITE,
    READ,
};


// Device reset methods.
enum
{
    PUC_RESET = 1 << 0, // Power up clear (i.e., a "soft") reset.
    RST_RESET = 1 << 1, // RST/NMI (i.e., "hard") reset.
    VCC_RESET = 1 << 2, // Cycle Vcc (i.e., a "power on") reset.
};

#define ALL_RESETS (PUC_RESET + RST_RESET + VCC_RESET)

// FLASH erase type.
enum
{
    ERASE_SEGMENT,  // Erase a segment.
    ERASE_MAIN,     // Erase all MAIN memory.
    ERASE_ALL,      // Erase all MAIN and INFORMATION memory.
};

// Configurations to set with MSP430_Configure.
enum
{
    VERIFICATION_MODE,  // Verify data downloaded to FLASH memories.
    EMULATION_MODE,	    // 4xx emulation mode.
    CLK_CNTRL_MODE,     // Clock control mode (on emulation stop).
    MCLK_CNTRL_MODE,    // Module Clock control mode (on emulation stop).
    FLASH_TEST_MODE,    // Flash test mode for Automotive Devices
};


// Definitions for the Flash Marginal Read Test
enum
{
    FLASH_MARGINAL_READ_OFF,
    FLASH_MARGINAL_READ_0,
    FLASH_MARGINAL_READ_1,
};

// File types.
enum
{
    FILETYPE_AUTO,      // Auto detect.
    FILETYPE_TI_TXT,    // TI text.
    FILETYPE_INTEL_HEX, // Intel hex.
};

// Error codes.
enum
{
    NO_ERR,                 // No error. *** This must be the first entry of the enumeration. ***
    INITIALIZE_ERR,         // Could not initialize device interface.
    CLOSE_ERR,              // Could not close device interface.
    PARAMETER_ERR,          // Invalid parameter(s).
    NO_DEVICE_ERR,          // Could not find device (or device not supported).
    DEVICE_UNKNOWN_ERR,     // Unknown device.
    READ_MEMORY_ERR,        // Could not read device memory.
    WRITE_MEMORY_ERR,       // Could not write device memory.
    READ_FUSES_ERR,         // Could not read device configuration fuses.
    CONFIGURATION_ERR,      // Incorrectly configured device; device derivative not supported.
    VCC_ERR,                // Could not set device Vcc.
    RESET_ERR,              // Could not reset device.
    PRESERVE_RESTORE_ERR,   // Could not preserve/restore device memory.
    FREQUENCY_ERR,          // Could not set device operating frequency.
    ERASE_ERR,              // Could not erase device memory.
    BREAKPOINT_ERR,         // Could not set device breakpoint.
    STEP_ERR,               // Could not single step device.
    RUN_ERR,                // Could not run device (to breakpoint).
    STATE_ERR,              // Could not determine device state.
    EEM_OPEN_ERR,           // Could not open Enhanced Emulation Module.
    EEM_READ_ERR,           // Could not read Enhanced Emulation Module register.
    EEM_WRITE_ERR,          // Could not write Enhanced Emulation Module register.
    EEM_CLOSE_ERR,          // Could not close Enhanced Emulation Module.
    FILE_OPEN_ERR,          // File open error.
    FILE_DETECT_ERR,        // File type could not be identified.
    FILE_END_ERR,           // File end error.
    FILE_IO_ERR,            // File input/output error.
    FILE_DATA_ERR,          // File data error.
    VERIFY_ERR,             // Verification error.
    BLOW_FUSE_ERR,          // Could not blow device security fuse.
    FUSE_BLOWN_ERR,         // Security Fuse has been blown
    INTEL_HEX_CODE_ERR,     // Error within Intel Hex file
    WRITE_REGISTER_ERR,     // Could not write device Register
    INVALID_ERR,            // Invalid error number. *** This must be the last entry of the enumeration. ***
};
    
#ifdef __cplusplus
extern "C" {
#endif

// Functions. -----------------------------------------------------------------

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Initialize(CHAR* port, LONG* version);

Description:
 Initialize the interface.

Parameters:
 port:    Interface port reference (application specific).
 version: The version number of the MSP430 DLL is returned (if version is not NULL).

Returns:
 STATUS_OK:    The interface was initialized.
 STATUS_ERROR: The interface was not initialized.

Error codes:
 INITIALIZE_ERR

Notes:
 1. *** This function must be called first. ***
 2. *** MSP430_Vcc() must be called second (after MSP430_Initialize() is called). ***
 3. This function calls user-supplied function: HIL_Initialize(port)
*/
STATUS_T WINAPI MSP430_Initialize(CHAR* port, LONG* version);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Close(LONG vccOff);

Description:
 Close the interface.

Parameters:
 vccOff: Turn off the device Vcc (0 volts) if TRUE.

Returns:
 STATUS_OK:    The interface was closed.
 STATUS_ERROR: The interface was not closed.

Error codes:
 CLOSE_ERR

Notes:
 1. If called, this function must be called last.
 2. This function calls user-supplied function: HIL_Close()
*/
STATUS_T WINAPI MSP430_Close(LONG vccOff);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Identify(CHAR* buffer, LONG count, LONG setId);

Description:
 Identify the device, and obtain device information.

Parameters:
 buffer: A pointer to a buffer where the device identity and information is stored. The values are returned
         in the buffer at the following offsets:
          0: The value 0xaa55 [so one can determine how to unpack the subsequent fields] (WORD).
          2: Device identification (WORD). Refer to the "Devices" enumeration
          4: Identification string (CHAR[32])
         36: MAIN MEMORY (FLASH) starting address (WORD). Ending address: 0xffff
         38: INFORMATION MEMORY (FLASH) starting address (WORD). Ending address: 0x10ff
         40: RAM ending address (WORD). Starting address: 0x0200
         42: Number of breakpoints (WORD)
         44: Emulation level (WORD)
         46: Clock control level (WORD)
         48: LCD starting address (WORD)
         50: LCD ending address (WORD)
         52: Vcc minimum during operation [mVolts] (WORD)
         54: Vcc maximum during operation [mVolts] (WORD)
         56: Device has TEST/VPP (WORD)
         58: RAM starting address (WORD).
         60: RAM2 starting address (WORD).
         62: RAM2 ending address (WORD).
         64: INFO ending address (WORD).
         66: Main ending address (LONG).
 count: The number of bytes to return in the buffer (i.e., the buffer size).
 setId: If setId is not DEVICE_UNKNOWN, the device is set to setId. Otherwise the device is determined.

Returns:
 STATUS_OK:    The device identify and information was obtained.
 STATUS_ERROR: The device identity and information was not obtained.

Error codes:
 PARAMETER_ERR
 NO_DEVICE_ERR
 READ_MEMORY_ERR
 READ_FUSES_ERR
 CONFIGURATION_ERR

Notes:
 1. It is possible to successfully (i.e. return status: STATUS_OK) identify an "UNKNOWN DEVICE".
 2. Use setId and MSP430_State(state, TRUE, FALSE) to stop a running device (without resetting it).
 3. Following MSP430_Identify():
    - the device is reset, is not executing, and is under JTAG control.
    - the JTAG interface is enabled and the JTAG signals are negated.
    - Vpp is set to the level specified in HIL_Open().
*/
STATUS_T WINAPI MSP430_Identify(CHAR* buffer, LONG count, LONG setId);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Device(LONG deviceId, CHAR* buffer, LONG count);

Description:
 Obtain the device information.

Parameters:
 deviceId: An index (from zero to DEVICE_END) used to specify the device.
 buffer: A pointer to a buffer where the device identity and information is stored. The values are returned
         in the buffer at the following offsets:
          0: The value 0xaa55 [so one can determine how to unpack the subsequent fields] (WORD).
          2: Device identification (WORD). Refer to the "Devices" enumeration
          4: Identification string (CHAR[32])
         36: MAIN MEMORY (FLASH) starting address (WORD). Ending address: 0xffff
         38: INFORMATION MEMORY (FLASH) starting address (WORD). Ending address: 0x10ff
         40: RAM ending address (WORD). Starting address: 0x0200
         42: Number of breakpoints (WORD)
         44: Emulation level (WORD)
         46: Clock control level (WORD)
         48: LCD starting address (WORD)
         50: LCD ending address (WORD)
         52: Vcc minimum during operation [mVolts] (WORD)
         54: Vcc maximum during operation [mVolts] (WORD)
         56: Device has TEST/VPP (WORD)
         58: RAM starting address (WORD).
         60: RAM2 starting address (WORD).
         62: RAM2 ending address (WORD).
         64: INFO ending address (WORD).
         66: Main ending address (LONG).
 count: The number of bytes to return in the buffer (i.e., the buffer size).

Returns:
 STATUS_OK:    The device identify and information was obtained.
 STATUS_ERROR: The device identity and information was not obtained.

Error codes:
 PARAMETER_ERR

Notes:
 1. This function can be used to determine the devices supported by the driver; a device
    identification of NUMofDevices is returned to indicate that there are no additional devices.
*/
STATUS_T WINAPI MSP430_Device(LONG deviceId, CHAR* buffer, LONG count);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Configure(LONG mode, LONG value);

Description:
 Configure the mode(s) of the device and/or the software.

Parameters:
 mode:     The mode to be configured.
 value:    The mode value.

Returns:
 STATUS_OK:    The mode was configured.
 STATUS_ERROR: The mode was not configured.

Error codes:
 PARAMETER_ERR

Notes:
 1. Specify a mode of VERIFICATION_MODE to verify data downloaded into FLASH memories.
*/
STATUS_T WINAPI MSP430_Configure(LONG mode, LONG value);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_VCC(LONG voltage);

Description:
 Set the device Vcc pin to voltage/1000 volts.

 *** The sample HIL.dll that interfaces to the MSP430 via the FET Interface Module connected
 *** to the parallel port of the PC >>> DOES NOT <<< implement the functionality required to
 *** set the device Vcc pin to voltage/1000 volts.

Parameters:
 voltage: The device Vcc pin is set to voltage/1000 volts.

Returns:
 STATUS_OK:    The Vcc was set to voltage.
 STATUS_ERROR: The Vcc was not set to voltage.

Error codes:
 PARAMETER_ERR
 VCC_ERR

Notes:
 1. This function calls user-supplied function: HIL_VCC(voltage).
 2. A "voltage" of zero (0) turns off voltage to the device.
 3. *** This function must be called second (after MSP430_Initialize() is called). ***
*/
STATUS_T WINAPI MSP430_VCC(LONG voltage);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Reset(LONG method, LONG execute, LONG releaseJTAG);

Description:
 Reset the device using the specified method(s). Optionally start device execution,
 and release the JTAG control signals.

Parameters:
 method:      The bit mask specifying the method(s) to use to reset the device:
                PUC:     The device is reset using PUC (i.e., a "soft" reset).
                RST_NMI: The device is reset using the RST/NMI pin (i.e., a "hard" reset).
                VCC:     The device is reset by cycling power to the device.
 execute:     Start device execution (when TRUE).
 releaseJTAG: Release the JTAG control signals (when TRUE). execute must be TRUE.

Returns:
 STATUS_OK:    The device was reset (and optionally started [and JTAG control released])
 STATUS_ERROR: The device was not reset (and optionally started [and JTAG control released]).

Error codes:
 PARAMETER_ERR
 RESET_ERR
 RUN_ERR
 NO_DEVICE_ERR

Notes:
 1. It is possible to combine reset methods. The methods are applied in the following order:
    PUC then RST_NMI then VCC. If a reset operation fails, the next reset method (if any) is applied.
 2. Following reset by RST/NMI and/or VCC, a PUC is automatically executed to reset the device in
    such a way that it does not begin to execute a resident program (or garbage).
 3. The device registers are updated (for read using MSP430_Registers()) following MSP430_Reset().
*/
STATUS_T WINAPI MSP430_Reset(LONG method, LONG execute, LONG releaseJTAG);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Erase(LONG type, LONG address, LONG length);

Description:
 Erase the device FLASH memory.

Parameters:
 type: ERASE_SEGMENT: Erase the segment containing 'address'.
       ERASE_MAIN:    Erase the Main memory.
       ERASE_ALL:     Erase the Main and Information memories.
 address:             Starting address of erase check operation. Must be word aligned.
 length:              Length of erase check operation (even number of bytes).

Returns:
 STATUS_OK:    The device FLASH memory was erased.
 STATUS_ERROR: The device FLASH memory was not erased.

Error codes:
 PARAMETER_ERR
 PRESERVE_RESTORE_ERR
 FREQUENCY_ERR
 ERASE_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. Use MSP430_Read_Memory() to determine the address of a failed erase operation.
 3. If address+length extends beyond the segment containing address, intermediate segments are erased
    (and checked).
 4. Following erasure of the FLASH, the device oscillator will be set to approx. 2.1MHz.
*/
STATUS_T WINAPI MSP430_Erase(LONG type, LONG address, LONG length);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Memory(LONG address, CHAR* buffer, LONG count, LONG rw);

Description:
 Read and write the device memory. "Device memory" includes the Special Function Registers (i.e., peripheral
 registers), RAM, Information (FLASH) memory, and Main (FLASH) memory.

Parameters:
 address: The starting address of the device memory to be read or written.
 buffer:  The buffer into which device memory is read, or from which device memory is written.
 count:   The number of bytes of device memory read or written.
 rw:      Specify a read (READ) or write (WRITE) operation.

Returns:
 STATUS_OK:    The memory operation encountered no errors.
 STATUS_ERROR: The memory operation encountered errors.

Error codes:
 PARAMETER_ERR
 READ_MEMORY_ERR
 WRITE_MEMORY_ERR
 PRESERVE_RESTORE_ERR
 FREQUENCY_ERR

Notes:
 1. A data write operation can be verified by performing a data read operation followed by a comparison at
    the application level.
 2. Usually, the FLASH memory must be erased before the write to FLASH memory operation is performed.
    *** The write to FLASH memory operation DOES NOT erase the FLASH before the operation is performed. ***
 3. Following a write to FLASH, the device oscillator will be set to approx. 2.1MHz.
*/
STATUS_T WINAPI MSP430_Memory(LONG address, CHAR* buffer, LONG count, LONG rw);

#define MSP430_Read_Memory(ADDRESS, BUFFER, COUNT) MSP430_Memory(ADDRESS, BUFFER, COUNT, READ)
#define MSP430_Write_Memory(ADDRESS, BUFFER, COUNT) MSP430_Memory(ADDRESS, BUFFER, COUNT, WRITE)

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Secure(void);

Description:
 The device is secured (i.e., the JTAG security fuse is blown).

 *** The sample HIL.dll that interfaces to the MSP430 via the FET Interface Module connected
 *** to the parallel port of the PC >>> DOES NOT <<< implement the functionality required to
 *** blow the device security fuse.

Parameters:

Returns:
 STATUS_OK:    The device was secured.
 STATUS_ERROR: The device was not secured.

Error codes:
 DEVICE_UNKNOWN_ERR
 BLOW_FUSE_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. Once a device is secure, further communications via JTAG is not possible.
 3. This function requires that HIL_TEST_VPP(), HIL_VCC(), and HIL_VPP() be implemented,
    and HIL_JTAG_IR() must support shifting instructions in via TDO.
 4. After MSP430_Secure(), the device is reset with Vcc set to ((MaxOp + MinOp) / 2) volts (*)
    and Vpp set to 0 volts (*). (*) Implementation dependent.
*/
STATUS_T WINAPI MSP430_Secure(void);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_ReadOutFile(LONG wStart, LONG wLength, LPTSTR lpszFileName, LONG iFileType);

Description:
 Read the specified range of device memory, and write it to the specified file.

Parameter:
 wStart:       The starting address of the device memory to read.
 wLength:      The length of the device memory to read (even number of bytes).
 lpszFileName: Pointer to the filename into which the read device memory is written.
 iFileType:    The type of the file into which the read device memory is written:
                TI text: FILETYPE_TI_TXT
                Intel hex: FILETYPE_INTEL_HEX

Returns:
 STATUS_OK:    The device memory was read and written to the specified file.
 STATUS_ERROR: The device memory was not read or written to the specified file.

Error codes:
 PARAMETER_ERR
 READ_MEMORY_ERR
 FILE_IO_ERR

Notes:
*/
#ifndef uController
STATUS_T WINAPI MSP430_ReadOutFile(LONG wStart, LONG wLength, LPTSTR lpszFileName, LONG iFileType);
#endif

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_ProgramFile(LPTSTR File, LONG eraseType, LONG verifyMem);

Description:
 The contents of the specified file are input, and then written to the device.
 The device is optionally erased prior to being written. The device is optionally
 verified after being written.

Parameter:
 File:         Pointer to filename.
 eraseType:    Type of device erasure:
                ERASE_ALL: Erase all Main and Information memories.
                ERASE_MAIN: Erase all Main memories.
 verifyMem:    Verify the device if TRUE.

Returns:
 STATUS_OK:    The file contents were input and were written to the device.
 STATUS_ERROR: The file contents were not input or were not written to the device.
 
Error codes:
 DEVICE_UNKNOWN_ERR
 PARAMETER_ERR
 FILE_END_ERR
 FILE_DETECT_ERR
 FILE_IO_ERR
 FILE_DATA_ERR
 PRESERVE_RESTORE_ERR
 FREQUENCY_ERR
 ERASE_ERR
 READ_MEMORY_ERR
 WRITE_MEMORY_ERR
 VERIFY_ERR
 ERR_INTEL_HEX_CODE
 
Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. The file type must be one of TI text or Intel hex. 
 3. verifyMem operation does not read the device memory and compare it to the file.
    Instead, this operation computes a checksum for the device memory, and
    then computes a checksum for the file, and finally compares the two checksums.
 4. Following a write to FLASH, the device oscillator will be set to approx. 2.1MHz.
 5. Following verifyMem, the device is reset.
*/
#ifndef uController
STATUS_T WINAPI MSP430_ProgramFile(LPTSTR File, LONG eraseType, LONG verifyMem);
#endif

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_VerifyFile(LPTSTR File);

Description:
 Compare the MSP430 memory and the contents of the specified file.

Parameter:
 File:         Pointer to filename.

Returns:
 STATUS_OK:    The file contents were input and compare with the device memory.
 STATUS_ERROR: The file contents were not input or did not compare with the device memory.

Error codes:
 PARAMETER_ERR
 FILE_END_ERR
 FILE_DETECT_ERR
 FILE_IO_ERR
 FILE_DATA_ERR
 VERIFY_ERR
 ERR_INTEL_HEX_CODE

Notes:
 1. The file type must be one of TI text or Intel hex. 
 2. This function does not read the device memory and compare it to the file.
    Instead, this operation computes a checksum for the device memory, and
    then computes a checksum for the file, and finally compares the two checksums.
 3. The device is reset following this operation.
*/
#ifndef uController
STATUS_T WINAPI MSP430_VerifyFile(LPTSTR File);
#endif

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_VerifyMem(LONG StartAddr, LONG Length, CHAR *DataArray);

Description:
 Compare the MSP430 memory and the specified data.

Parameter:
 StartAddr:    Start address of memory to be compared (must be even).
 Length:       Number of bytes to be compared (must be even).
 *DataArray:   Pointer to data array.

Returns:
 STATUS_OK:    The device memory and data compare.
 STATUS_ERROR: The device memory and data do not compare.

Error codes:
 PARAMETER_ERR
 VERIFY_ERR

Notes:
 1. This function does not read the specified memory region and compare it to the data.
    Instead, this function computes a checksum for the specified memory region, and
    then computes a checksum for the data, and finally compares the two checksums.
 2. The device is reset following this operation.
*/
STATUS_T WINAPI MSP430_VerifyMem(LONG StartAddr, LONG Length, CHAR *DataArray);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_EraseCheck(LONG StartAddr, LONG Length);

Description:
 Verify that the specified memory range is erased.

Parameter:
 StartAddr:    Start address of memory to be verified (must be even).
 Length:       Number of BYTEs to be verified (must be even).

Returns:
 STATUS_OK:    The device memory in the specified range is erased.
 STATUS_ERROR: The device memory in the specified range is not erased.

Error codes:
 PARAMETER_ERR
 VERIFY_ERR

Notes:
 1. This function does not read the specified memory region and compare it to erased data.
    Instead, this function computes a checksum for the specified memory region, and
    then computes a checksum for erased data, and finally compares the two checksums.
 2. The device is reset following this operation.
*/
STATUS_T WINAPI MSP430_EraseCheck(LONG StartAddr, LONG Length);

/* ----------------------------------------------------------------------------
Function:
 LONG WINAPI MSP430_Error_Number(void);

Description:
 Determine the number of the error when an MSP430_xxx() function returns STATUS_ERROR.

Parameter:

Returns:
 The number of the last error.

Notes:
 1. The error number is reset (to NO_ERR) after the error number is returned.
*/
LONG WINAPI MSP430_Error_Number(void);

/* ----------------------------------------------------------------------------
Function:
 const CHAR* WINAPI MSP430_Error_String(LONG errorNumber);

Description:
 Determine the string associated with errorNumber.

Parameter:
 errorNumber: Error number.

Returns:
 The string associated with errorNumber.

Notes:
*/
#ifndef uController
const CHAR* WINAPI MSP430_Error_String(LONG errorNumber);
#endif

#ifdef __cplusplus
}
#endif

#endif // _MSP430_H_
