/*========================================================================*\
|                                                                          |
| MSP430.h                                                                 |
|                                                                          |
| This file contains the DLL function headers and definitions              |
| The following definitions support the debugging functions of the         |
| MSP430 DLL. The debugging functions of the DLL are made available only   |
| to third party tool developers.                                          |
|--------------------------------------------------------------------------|
| Project:              MSP430 JTAG interfcae                              |
| Developed using:      MS Visual C++ 5.0                                  |
|--------------------------------------------------------------------------|
| Version:              1.1.5                                              |
| Initial Version:      20 / 07 / 02                                       |
| Last Change:          01 / 20 / 04                                       |
|--------------------------------------------------------------------------|
| Version history:                                                         |
| Version: 1.1.5                                                           |
| 01/21/2004 UPSF moved MSP430_Configure enum to MSP430.h                  |
|                                                                          |
|--------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments                                       |
\*========================================================================*/

/*------------------------------------------------------------------------*\
| Remarks:                                                                 |
|                                                                          |
\*------------------------------------------------------------------------*/


#ifndef _MSP430_DEBUG_H_
#define _MSP430_DEBUG_H_

// #includes. -----------------------------------------------------------------

#include "MSP/MSP430.h"

// Temporary work-around to #defines in stdconst.h (and others)
#undef R0
#undef R1
#undef R2
#undef R3
#undef R4
#undef R5
#undef R6
#undef R7
#undef R8
#undef R9
#undef R10
#undef R11
#undef R12
#undef R13
#undef R14
#undef R15
#undef PC
#undef SP
#undef SR
#undef CG1
#undef CG2
#undef VCC

// #defines. ------------------------------------------------------------------


// Device registers.
enum
{
	R0, R1, R2,	R3,	R4,	R5,	R6,	R7,	R8,	R9,	R10, R11, R12, R13,	R14, R15,
	PC = 0, SP,	SR, CG1 = 2, CG2,
};

// Device registers masks.
#define MASKREG(REG) (1 << REG)
#define ALL_REGS 0xffff
	
// Run modes.
enum
{
	FREE_RUN = 1,		// Run the device. Set breakpoints (if any) are disabled.
	SINGLE_STEP,		// A single device instruction is executed. Interrupt processing is supported.
	RUN_TO_BREAKPOINT,	// Run the device. Set breakpoints (if any) are enabled.
};

// State modes.
enum
{
	STOPPED,				// The device is stopped.
	RUNNING,				// The device is running or is being single stepped.
	SINGLE_STEP_COMPLETE,	// The device is stopped after the single step operation is complete.
	BREAKPOINT_HIT,			// The device is stopped as a result of hitting an enabled breakpoint.
};

// One of the following enumerations is returned in device.emulation
enum
{
	EMEX_NONE,		// Device has no Emex module.
	EMEX_LOW,		// Device Emex module has two breakpoints.
	EMEX_MEDIUM,	// Device Emex module has three breakpoints and range comparison.
	EMEX_HIGH		// Device Emex module has eight breakpoints, range comparison, state storage, and trigger sequencer,
};

// EEM (Extended) General Clock Control.

// The EEM (Extended) General Clock Control allows one to configure the behavior of the system clocks
// (i.e., ACLK, MCLK, SMCLK, TACLK) when the device is stopped by JTAG (i.e., when the device is not
// running or being single stepped). The default behavior of the clocks is to continue to function
// when the device is stopped by JTAG.

// Use function MSP430_Configure(CLK_CNTRL_MODE, <bits>); to configure the bits of the EEM (Extended)
// General Clock Control register.

// One of the following enumerations is returned in device.clockControl
enum
{
	GCC_NONE,		// Device has no clock control. The system clock continue to function when the device is stopped by JTAG.
	GCC_STANDARD,	// Device has General Clock Control register.
	GCC_EXTENDED,	// Device has Extended General Clock Control register and Module Clock Control register 0.
};

// Bits of the EEM General Clock Control register (F41x).
#define TCE_SMCLK			0x0001 // Clock SMCLK with TCLK. See Note 1.
#define ST_ACLK				0x0002 // Stop ACLK
#define ST_SMCLK			0x0004 // Stop SMCLK
#define TCE_MCLK			0x0008 // Clock functional MCLK with TCLK. See Note 1.
#define JT_FLLO				0x0010 // Switch off FLL
#define ST_TACLK			0x0020 // Stop TACLK

// Bits of the EEM Extended General Clock Control register (F43x/F44x).
#define ECLK_SYN			0x0001 // Emulation clock synchronization. See Note 1.
//#define ST_ACLK			0x0002 // Stop ACLK. See Note 2.
//#define ST_SMCLK			0x0004 // Stop SMCLK. See Note 2.
#define ST_MCLK				0x0008 // Stop MCLK
//#define JT_FLLO			0x0010 // Switch off FLL. See Note 2.
#define FORCE_SYN			0x0040 // Force JTAG synchronization. See Note 1.

// Note 1: Not recommended for use with MSP430_Configure(CLK_CNTRL_MODE, <bits>);
// Note 2: ST_ACLK, ST_SMCLK, and JT_FLLO *are* supported by the EEM Extended General Clock Control register;
//		   Their definitions were commented-out so as not to conflict with the identical definitions for the
//		   EEM General Clock Control register.

// In addition to the above EEM Extended General Clock Control register features, the MSP430F449
// supports four emulation modes (EMU_MODE_F4xx_xxx) using the EEM General Control register.

// Use function MSP430_Configure(EMULATION_MODE, <bits>); to configure the bits of the EEM General Control
// register.
// Note 1. Only the EMU_MODE_F4xx_xxx bits are supported by EMULATION_MODE.
// Note 2. MSP430_Configure(EMULATION_MODE, <bits>); will reset the device with PUC in order to change the
//         emulation mode.
 
// Bits of the EEM General Control register.
#define	EEM_EN				0x0001 // Do not use with MSP430_Configure(EMULATION_MODE, <bits>);
#define	CLEAR_STOP			0x0002 // Do not use with MSP430_Configure(EMULATION_MODE, <bits>);
#define EMU_CLK_EN			0x0004 // Do not use with MSP430_Configure(EMULATION_MODE, <bits>);
#define EMU_FEAT_EN			0x0008 // Do not use with MSP430_Configure(EMULATION_MODE, <bits>);
// The actual name of this bit is STOPPED.
#define	EEM_STOPPED			0x0080 // Do not use with MSP430_Configure(EMULATION_MODE, <bits>);
#define EMU_MODE_F44X_100	0x0000 // (Emulate) F44x 100 pins. This is the normal device mode.
#define EMU_MODE_F43X_100	0x4000 // Emulate F43x 100 pins.
#define EMU_MODE_F4XX_64	0x5000 // Emulate F4xx 64 pins.
#define EMU_MODE_F4XX_80	0x6000 // Emulate F4xx 80 pins.

#ifdef __cplusplus
extern "C" {
#endif

// Functions. -----------------------------------------------------------------

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Registers(LONG registers[], LONG mask, LONG rw);

Description:
 Read and write the device registers.

Parameters:
 registers: The destination of registers read from the device (rw = READ),
			and the source of registers written to the device (rw = WRITE).
 mask:		A bit-mask which identifies which registers are read/written
			(Bit 0: R0, Bit 1: R1, and so on).
 rw:		Specify a read (READ) or write (WRITE) operation.

Returns:
 STATUS_OK:	   The registers were read or written.
 STATUS_ERROR: The registers were not read or written.

Error codes:
 PARAMETER_ERR

Notes:
 1. It is not possible to read and write registers CG1 and CG2.
 2. This function does not read and write the actual device registers. Rather, copies of the device
    registers are read and written. The register copies are updated (for read) after a device reset,
	and after MSP430_State() indicates BREAKPOINT_HIT, SINGLE_STEP_COMPLETE, or STOPPED (after
	stopping a running device [stop = TRUE]). The register copies are written to the device prior to
	MSP430_Run() with FREE_RUN, SINGLE_STEP, and RUN_TO_BREAKPOINT.
*/
STATUS_T WINAPI MSP430_Registers(LONG registers[], LONG mask, LONG rw);

#define MSP430_Read_Registers(REGISTERS, MASK) MSP430_Registers(REGISTERS, MASK, READ)
#define MSP430_Write_Registers(REGISTERS, MASK) MSP430_Registers(REGISTERS, MASK, WRITE)

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Breakpoint(LONG number, LONG address);

Description:
 An Instruction Fetch breakpoint is set on address "address". Breakpoint block number "number" is used.

Parameters:
 number:  The number of the breakpoint block to use (>= 0).
 address: The address of the Instruction Fetch breakpoint.

Returns:
 STATUS_OK:	   The breakpoint was set.
 STATUS_ERROR: The breakpoint was not set.

Error codes:
 PARAMETER_ERR
 BREAKPOINT_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. Setting a breakpoint in an invalid breakpoint block will return STATUS_ERROR. This mechanism can be
    employed to determine how many breakpoint blocks a device supports. Or reference device.nBreakpoints.
 3. A breakpoint block can be disabled by setting its breakpoint address to zero (0x0).
 4. It is the responsibility of the application to maintain what breakpoints (if any) are set in what
    breakpoint blocks (as it's not possible to read the breakpoint address of a breakpoint block).
 5. A breakpoint set on odd address N will be set on even address N-1.
 6. A breakpoint is enabled only prior to RUN_TO_BREAKPOINT.
 7. Hitting a breakpoint does not clear/disable it; it must be cleared/disabled explicitly.
*/
STATUS_T WINAPI MSP430_Breakpoint(LONG number, LONG address);

#define MSP430_Clear_Breakpoint(NUMBER) MSP430_Breakpoint(NUMBER, 0)

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_Run(LONG mode, LONG releaseJTAG);

Description:
 Run the device using the specified mode. JTAG control signals are optionally released.

Parameters:
 mode:		  The specified run mode:
			   FREE_RUN:		  Run the device. Set breakpoints (if any) are disabled.
			   SINGLE_STEP:		  A single device instruction is executed. Interrupt processing is supported.
			   RUN_TO_BREAKPOINT: Run the device. Set breakpoints (if any) are enabled.
 releaseJTAG: The JTAG control signals are released when TRUE.

Returns:
 STATUS_OK:	   The run operation encountered no errors.
 STATUS_ERROR: The run operation encountered errors.

Error codes:
 DEVICE_UNKNOWN_ERR
 STEP_ERR
 RUN_ERR
 BREAKPOINT_ERR
 PARAMETER_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. Use MSP430_State() to determine the device state after an MSP430_Run() operation.
 3. Use MSP430_State() to update the device registers before using MSP430_Registers() to read the
    device registers.
*/
STATUS_T WINAPI MSP430_Run(LONG mode, LONG releaseJTAG);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_State(LONG* state, LONG stop, LONG* pCPUCycles);

Description:
 Determine the state of the device following an MSP430_Run() operation. The device is optionally stopped.
 During single step operations, a count of CPU cycles executed is maintained.

Parameters:
 state:		 The device state:
			  STOPPED:				The device is stopped.
			  RUNNING:				The device is running or is being single stepped.
			  SINGLE_STEP_COMPLETE: The device is stopped after completing the single step operation.
			  BREAKPOINT_HIT:		The device is stopped as a result of hitting an enabled breakpoint.
 stop:		 The device is stopped when TRUE.
 pCPUCycles: The cumulative number of CPU cycles executed during single step (or -1 if invalid).

Returns:
 STATUS_OK:	   The device state was determined.
 STATUS_ERROR: The device state was not determined.

Error codes:
 PARAMETER_ERR
 STATE_ERR
 STEP_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. To determine the device state, the JTAG control signals are required. If the JTAG control signals were
	released, the signals are reconnected. If the state is RUNNING and the JTAG control signals had been
	released and stop is FALSE, the JTAG control signals are released after determining the state.
 3. The number of CPU cycles executed is zeroed when the device is reset (MSP430_Reset()), and when the
    device is run (MSP430_Run()) with FREE_RUN and RUN_TO_BREAKPOINT.
 4. Use MSP430_State() to update the device registers before using MSP430_Registers() to read the
    device registers.
*/
STATUS_T WINAPI MSP430_State(LONG* state, LONG stop, LONG* pCPUCycles);

// Enhanced Emulation Module functions ----------------------------------------

/* ----------------------------------------------------------------------------

   The following functions implement a very low-level interface to the Enhanced Emulation Module (EEM).
   The EEM provides advanced debugging facilities (including advanced breakpoints, trace buffers, etc.).
   Please contact Texas Instruments to obtain the detailed information and device specifications required
   to make use of this mechanism.

   ---------------------------------------------------------------------------- */

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_EEM_Open(void);

Description:
 Open the Enhanced Emulation Module (EEM).

Parameters:

Returns:
 STATUS_OK:	   The EEM was opened.
 STATUS_ERROR: The EEM was not opened.

Error codes:
 DEVICE_UNKNOWN_ERR
 EEM_OPEN_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
*/
STATUS_T WINAPI MSP430_EEM_Open(void);

/* ----------------------------------------------------------------------------
Function:
 LONG WINAPI MSP430_EEM_Read_Register(LONG address);

Description:
 Read the EEM register at address "address".

Parameters:
 address: The address of the EEM register read.

Returns:
 The value read from the EEM register.

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. No status is returned from this function; use MSP430_EEM_Read_Register_Test() if a status is desired.
 3. MSP430_EEM_Open() will be called if required.
*/
LONG WINAPI MSP430_EEM_Read_Register(LONG address);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_EEM_Read_Register_Test(LONG address, LONG* value);

Description:
 Read the EEM register at address "address", and store the read value at "value".

Parameters:
 address: The address of the EEM register read.
 value:	  The address of where the read value is stored.

Returns:
 STATUS_OK:	   The EEM read register operation encountered no errors.
 STATUS_ERROR: The EEM read register operation encountered errors.

Error codes:
 PARAMETER_ERR
 EEM_OPEN_ERR
 EEM_READ_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. MSP430_EEM_Open() will be called if required.
*/
STATUS_T WINAPI MSP430_EEM_Read_Register_Test(LONG address, LONG* value);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_EEM_Write_Register(LONG address, LONG value);

Description:
 Write the EEM register at address "address" with value "value".

Parameters:
 address: The address of the EEM register written.
 value:	  The value to be written into the EEM register.

Returns:
 STATUS_OK:	   The EEM write register operation encountered no errors.
 STATUS_ERROR: The EEM write register operation encountered errors.

Error codes:
 PARAMETER_ERR
 EEM_OPEN_ERR
 EEM_WRITE_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
 2. MSP430_EEM_Open() will be called if required.
*/
STATUS_T WINAPI MSP430_EEM_Write_Register(LONG address, LONG value);

/* ----------------------------------------------------------------------------
Function:
 STATUS_T WINAPI MSP430_EEM_Close(void);

Description:
 Close the Enhanced Emulation Module (EEM).

Parameters:

Returns:
 STATUS_OK:	   The EEM was closed.
 STATUS_ERROR: The EEM was not closed.

Error codes:
 DEVICE_UNKNOWN_ERR
 EEM_CLOSE_ERR

Notes:
 1. MSP430_Identify() must have been called prior to calling this function.
*/
STATUS_T WINAPI MSP430_EEM_Close(void);

#ifdef __cplusplus
}
#endif

#endif // _MSP430_DEBUG_H_
