#include <stdio.h>
#include <string.h>
#include "MSP/MSP430_Debug.h"
#include "MSP/MSP430.h"
#include "MSP/Devices.h"
#include "error.h"
#include "dat.h"
#include "fns.h"
#include "styx.h"
#include "arch/thread.h"
//#include "LowLevelFunc.h"			// Low level user functions



unsigned char prog_code[28] = {
0x31, 0x40, 0x00, 0x0A, 0xB2, 0x40, 0x80, 0x5A,
0x20, 0x01, 0xE2, 0xD3, 0x32, 0x00, 0xE2, 0xE3,
0x31, 0x00, 0x3F, 0x40, 0x50, 0xC3, 0x1F, 0x83,
0xFE, 0x23, 0xF9, 0x3F 
};

// even address, even count
unsigned char prog_code1[8] = {
0x31, 0x40, 0x00, 0x0A, 0xB2, 0x40, 0x80, 0x5A
};
// even address, odd count
unsigned char prog_code2[7] = {
0x20, 0x01, 0xE2, 0xD3, 0x32, 0x00, 0xE2
};
// odd address, even count
unsigned char prog_code3[4] = {
0xE3, 0x31, 0x00, 0x3F 
};
// odd address, odd count
unsigned char prog_code4[9] = {
0x40, 0x50, 0xC3, 0x1F, 0x83, 0xFE, 0x23, 0xF9, 0x3F 
};
// reset vector
unsigned char reset_vec[2] = {
0x00, 0xE0
};

LONG getDCOFreq(WORD DCO, WORD BCS1);

#define N_REGS              16
#define PC 0   // index of the PC register

extern const struct tsdeviceID sdeviceID[];
union DEVICE_T* device;
unsigned char idbuffer[64];
extern LONG deviceRegs[N_REGS];      // device registers

extern Dev mspdev;

enum{
  Qdir,
  Qregisters,
  Qmemory,
  Qstatus,
  Qcontrol
};

static const Dirtab
msptab[]={
  {".",    Qdir,   Qdir,  PDIR | 05},
  {"registers", Qregisters,  Qdir,  06},
  {"memory",    Qmemory, Qdir,  06},
  {"status",    Qstatus,  Qdir,  06},
  {"control",    Qcontrol, Qdir, 06}
};

extern Channel outChan;

Channel mspChan = CHANNELN(Fcall *, 4);
Channel mspfidChan = CHANNELN(iFid *, 4);


ErrorStruct msperrorstrt;

Thread mspdevT;

static void
mspread(iFid *c, void *buf, ushort n, vlong offset, ushort *len);

static void
mspwrite(iFid *c, void *buf, ushort n, vlong offset, ushort *len);

static void
mspwalk(iFid *c, char *name);

void
mspinit(Dev *d);


int char2hex(char c)
{
  if (!(( (c>='0') && (c <= '9')) || ((c >= 'A') && (c <= 'F'))  
            || ((c >= 'a') && (c <= 'f'))    )) {
    printf("Invalid arg %c for char2hex\n",c);
    exit(-1);
  }
   
  if ( (c>='0') && (c <= '9'))
    return (c - '0');

  if ((c >= 'A') && (c <= 'F')) 
    return (10 + c - 'A');

 if ((c >= 'a') && (c <= 'f')) 
    return (10 + c - 'a');

  return -1;
}

char hex2char(int h)
{
  if (!((h<16) && (h>=0))) {
    printf("Invalid arg: %d for hex2int\n",h);
    exit(-1);
  }

  if (h<10)
    return ('0'+h);
  else
    return ('A'+(h - 10));
}

/*
  This is the control loop for the msp device.
  It receives calls through the call pipe and dispatches them to the corresponding function.
*/

void mspmain() {
  //  Fid *f;
  static Fcall *rx;
  static Fcall *tx;
  static iFid *f;
  char cmd;
  void *buf;
  u_int16_t count;
  u_int64_t offset;
  u_int8_t index;
  char *name;
  long lvarx;
  int status;
  
  printf("mspmain: rcving on cpipe %p\n",&mspChan);
  
  
  threadBegin();  
  
   
  
  while(1) {
    // receive call from the call pipe
    if (chanrecv(&mspChan, &rx) < 0)
      threadReturn();
    printf("mspthread: recvd 1  request\n");
    if (chanrecv(&mspfidChan, &f) < 0)
      threadReturn();
    if (chanrecv(&mspChan, &tx) < 0)
      threadReturn();

    if (!(setjmp(msperrorstrt.estack))) 
    switch(rx->type) {
 
    case Tread:
     count = rx->ns.un.Twrf.count;
     offset = rx->ns.un.Twrf.offset;
     mspread(f, tx->ns.un.Rwrf.rdata + 1, count, offset, &(tx->ns.un.Rwrf.rcount));    
    break;
  
    case Twrite:
     count = rx->ns.un.Twrf.count;
     offset = rx->ns.un.Twrf.offset;
     buf = rx->ns.un.Twrf.data;
     mspwrite(f, buf, count, offset, &(tx->ns.un.Rwrf.rcount));
    break;
  
    case Twalk:
     name = rx->ns.un.Twcf.name;
     mspwalk(f, name);
     tx->ns.un.qid = f->qid;
    break;
  
    default:
     tx->type = Rerror;
     strcpy(tx->ename, "Invalid operation on mspdev");
   }
   else
   {
     tx->type = Rerror;
     strcpy(tx->ename, msperrorstrt.errstr);
   }

   if (chansend(&outChan, &tx) < 0)
      threadReturn();    


    // call the corresponding function

    // send the response out of the output pipe

    // return the buffer
  }
  threadFinish();

}

static void
mspread(iFid *f, void *vbuf, ushort n, vlong offset, ushort *len)
{
   char *tmp;
  int  slen;
  int status;
  LONG temp;
  int i;
  LONG deviceRegs[N_REGS];        // Device registers.
  LONG state;
  LONG pcpuCycles;
  volatile WORD pc;
  char *buf;
 // char memory_read[8];
  WORD memory_read[80];;
 // WORD memword[10];;

  
  buf = (char *) vbuf;
 

  fprintf(stderr, "mspread\n");
  
  if (f->qid.path & DMDIR)
    {
      dirread(f, (Dir *) vbuf, msptab, nelem(msptab), len, offset, &msperrorstrt);
      return;
    }
    
  switch(f->qid.path & 255) {
   
  case Qmemory:
   if  (MSP430_Read_Memory(offset, (BYTE*) memory_read, n) != STATUS_OK) {
     printf("Error: Unable to read memory\n");
     *len = 0;
     return;
   }
  memcpy(buf, memory_read, n);
  /*
  if  (MSP430_Read_Memory(offset, (BYTE*) buf, n) != STATUS_OK) {
     printf("Error: Unable to read memory\n");
     *len = 0;
     return;
   }
   */
 
 //status = MSP430_Read_Memory(0xE042, (BYTE*) memory_read, 8);
   
   *len = n;
   return;
   break;
   
  case Qregisters :

    /***********************************************************************\
    In this case, n gives the number of registers to write (either 1 or all)
    In the case of n, the offset gives the register number 
    \***********************************************************************/


   if (n == 4*N_REGS) {

   if (MSP430_Registers(deviceRegs,0xffff,READ) != STATUS_OK) {
     printf("Error: Unable to read registers.");
      return;
   }

   for(i=0;i<N_REGS;i++) {
     
     temp = deviceRegs[i];
     
     /**************************************************************************************\
      The order in which data is sent out is: HNib of B1, LNib of B1, HNib of B0, LNib of B0

     \**************************************************************************************/
     
     buf[4*i] = hex2char((temp >> 12) & 0xF); 
     buf[4*i+1] = hex2char((temp >> 8) & 0xF); 
     buf[4*i+2] = hex2char((temp >> 4) & 0xF);
     buf[4*i+3] = hex2char(temp & 0xF);
   }
  
   *len = 4*N_REGS;   
   return;
   } else {
     if (n == 4) {
       if (MSP430_Registers(deviceRegs, (1<<offset), READ) != STATUS_OK) {
          printf("Error: Unable to write registers.\n");
          *len = 0;
          return;
       }           
       temp = deviceRegs[offset];

     /* shouldn't this just be buf[0], buf[1], ...  ?    */
       buf[4*offset] = hex2char((temp >> 12) & 0xF); 
       buf[4*offset+1] = hex2char((temp >> 8) & 0xF); 
       buf[4*offset+2] = hex2char((temp >> 4) & 0xF);
       buf[4*offset+3] = hex2char(temp & 0xF);
       
       //   *n = 1;
       *len = 4;
       return;
     } else {
       printf("Error: Incorrect number of registers in write\n");
       *len = 0;
       return;
     }
   }
   break;  
   
  case Qstatus:
    //    sttime = nr_timer_milliseconds();
    if (offset>0) {
     *len = 0;
     return;
     break;
    }
     
    if (MSP430_State(&state, FALSE, &pcpuCycles) != STATUS_OK) {
      *len = 0;
      printf("Error: unable to get state.");
      return;
    }

   switch (state) {
    case RUNNING:
      buf[0] = 'r';
      break;
    case STOPPED:
      buf[0] = 's';
      break;
    case SINGLE_STEP_COMPLETE:
      buf[0] = 't';
      break;
    case BREAKPOINT_HIT:
      buf[0] = 'b';
      break;
    default:
      buf[0] = 'x';
      break;
    }
    *len = 1;
    return;
    break; 
    
  default:
    printf("Read from invalid file.\n");
    *len = 0;
    return;
  }
   
}

static void
mspwrite(iFid *f, void *vbuf, ushort n, vlong offset, ushort *len)
{
  u_int8_t *buf;
  LONG state;
  LONG pCPUCycles;
  int bkptno;
  LONG addr;
  volatile long status;
  long lvarx,cpucycl,reg;
  LONG deviceRegs[N_REGS];        // Device registers.
  int i;
  LONG temp;
  char* tempmemstore;
  LONG pcpuCycles;
  volatile WORD pc;
  volatile char buffer[152];
  unsigned char *ucptr;
  WORD tempw;
  int nosteps;
  int sttime,endtime;
  WORD memory_read[50];
  WORD memory_write[80];
    
  buf = (char *) vbuf;
  fprintf(stderr, "mspwrite\n");
  
  if (f->qid.path & DMDIR)
   {
     *len = 0;
     return;
   }
  
   /* local file */

  switch(f->qid.path & 255) {
  case Qcontrol:
  
   switch (buf[0]) {
   case 's':   // initialization 
   
     if (n != 1) {
       printf("Error: Incorrect length for step command.");
       *len = 0;
       return;
     }

     if (MSP430_State(&state, TRUE, &pCPUCycles) != STATUS_OK) {
        printf("Error: Unable to stop device.\n");
        *len = 0;
        return;
     }
     *len = 1;
     return;
     break;
     
   case 't': //single step 
     /* incorporate error handling!    */
  
     if (n != 1) {
       printf("Error: Incorrect length for init command.\n");
       *len = 0;
       return;
     }

     if (MSP430_Run(SINGLE_STEP, FALSE) != STATUS_OK) {
       printf("Error: Unable to single step.\n");
       *len = 0;
       return;
     }

     *len = 1;
     return;
     break;
  
  
   case 'c': //continue

     if (n != 1) {
       printf("Error: Incorrect length for init command.\n");
       *len = 0;
       return;
     }

     if (MSP430_Run(RUN_TO_BREAKPOINT, FALSE) != STATUS_OK ) {
       printf("Error: Unable to continue.\n");
       *len = 0;
       return;
     }
     
     *len = 1;
     return;
     break;

   case 'b': //breakpoint

     //     sttime = nr_timer_milliseconds();
     bkptno = char2hex(buf[1]); // this is a number between '0' an '9' hopefully
     
     nosteps = 0;

     addr =  (char2hex(buf[2]) << 12) + (char2hex(buf[3]) << 8) + (char2hex(buf[4]) << 4) + char2hex(buf[5]);

     status = STATUS_OK;

     if ((status = MSP430_Breakpoint(bkptno, addr)) != STATUS_OK) {
       printf("Unable to set breakpoint.\n");
       *len = 0;
       return;
     }

     *len = 6;
     return;
     break;
     
    case 'e': //erase 
     if (MSP430_Erase(ERASE_ALL, device->mainStart, 0x10000 - device->mainStart) != STATUS_OK) {
       printf("Error: Unable to erase flash.\n");
       *len = 0;
       return;
     }

     if (MSP430_EraseCheck(device->mainStart, 0x1100 - device->mainStart) != STATUS_OK) {
       printf("Error: Unable to erase flash.\n");
       *len = 0;
       return;
     }

     *len = 1;
     return;
     break;


    default:
     printf("Warning: invalid operation : %c\n",buf[0]);
     *len = 0;
     return;
  }
     
 case Qmemory:
 memcpy(memory_write, buf, n);
/*
  if  (MSP430_Write_Memory(offset, buf, n) != STATUS_OK) {
       printf("Error: Unable to write to memory.\n");
       *len = 0;
       return;
     }
  */
     
 if  (MSP430_Write_Memory(offset, memory_write, n) != STATUS_OK) {
       printf("Error: Unable to write to memory.\n");
       *len = 0;
       return;
     }
 
  printf("wrote to memory fine\n");
  
  /*
  status = MSP430_Read_Memory(0xE000, (BYTE*) memory_read, 100);
  if (status<0) {
   printf("MSP430_Erase failed. exiting\n");
   return;
  }
  */
  
  *len = n;
  return;
  break;
 
   
 case Qregisters:
 
   /************************************
    In this case, n gives the number of registers to write (either 1 or all)
    In the case of n, the offset gives the register number 
   *****************************************************/

   if (n == (4 * N_REGS)) { // 4 chars per register (2 chars per byte)
     for(i=0;i<N_REGS;i++) {
       temp = (char2hex(buf[4*i+0]) << 12) + (char2hex(buf[4*i+1]) << 8) + (char2hex(buf[4*i+2]) << 4) + char2hex(buf[4*i+3]);
       deviceRegs[i] = temp;
     }
     if (MSP430_Registers(deviceRegs,0xf,WRITE) != STATUS_OK) {
       printf("Error: Unable to write registers.\n");
       *len = 0;
       return;
     }
     else {
       *len = 4 * N_REGS;
       return;
     }
   } else {
     if (n == 4) {
       temp = (char2hex(buf[0]) << 12) + (char2hex(buf[1]) << 8) + (char2hex(buf[2]) << 4) + char2hex(buf[3]);
       deviceRegs[offset] = temp;
       if (MSP430_Registers(deviceRegs,(1<<offset),WRITE) != STATUS_OK) {
          printf("Error: Unable to write single register.\n");
          *len = 0;
          return;
       }
       else
        *len = 4;
        return;
     } else {      
       printf("Error: Incorrect number of registers in write\n");
       *len = 0;
       return;
     }
   }
   printf("should never get here really\n");
   return;
   break; 
  
 default:
  printf("Warning: invalid qid %d\n",f->qid.path & 255);
  *len = 0;
  return;
  
 }
}
 

//---------------------------------------------------------------------------------------
void mainthrd (void)
{
  volatile long status;
  unsigned char idbuffer[64];
  char memory_read[100];
//  unsigned char i;
  union DEVICE_T* device = (union DEVICE_T*) &idbuffer[0]; // Device information structure.
  long lvarx,cpucycl,reg;
  long mcu_state = 0;
  char buf[2];
  volatile char buf0[2];
  LONG deviceRegs[N_REGS];        // Device registers.
  int i;
  LONG DCOFreq;
  
  //  InitSystem();
  // threadBegin();
  
   status = MSP430_Initialize(0, &lvarx);
  // status = MSP430_VCC((LONG) 3.3 * 1000);
  if (status<0) {
   printf("MSP430_Initialize faile. exiting\n");
   return;
  }
  printf("MSP430_Initialize status=%ld\n",status);
 

  status = MSP430_Identify( &idbuffer[0], 64, FALSE);
    if (status<0) {
   printf("MSP430_Identify faile. exiting\n");
   return;
  }
   printf("MSP430_Identify status=%ld\n",status);  
  
  status = MSP430_Reset(VCC_RESET, FALSE, FALSE);
  if (status<0) {
   printf("MSP430_Reset faile. exiting\n");
   return;
  }
  printf("MSP430_Reset status = %ld\n",status);
 
  
 
  status = MSP430_Erase(ERASE_ALL, device->mainStart, 0x10000 - device->mainStart);
   if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_Erase STATUS = %ld\n",status);
    
  status = MSP430_EraseCheck(device->infoStart, 0x1100 - device->infoStart);
 if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_Erasechk status = %ld\n",status );
   
  status = MSP430_Write_Memory(0xE000, &prog_code1[0], 8);             // even address, even count
   if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem0 status = %ld\n",status);
  
    
  status = MSP430_Write_Memory(0xE000+8, &prog_code2[0], 7);           // even address, odd count
  if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem1 status = %ld\n",status);
   
  status = MSP430_Write_Memory(0xE000+8+7, &prog_code3[0], 4);         // odd address, even count
  if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem2 status = %ld\n",status);
     
  status = MSP430_Write_Memory(0xE000+8+7+4, &prog_code4[0], 9);       // odd address, odd count
 if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem3 status = %ld\n",status);
  
  status = MSP430_Read_Memory(0xE000, (BYTE*) memory_read, 80);
   if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
 
  status = MSP430_Write_Memory(0xFFFE, &reset_vec[0], 2);              // reset vector
 
  MSP430_Breakpoint(0, 0xE00E);
  
  mcu_state = 89;

  MSP430_Run(RUN_TO_BREAKPOINT,FALSE);
  printf("waiting for break ...\n");
  while((mcu_state != STOPPED) && (mcu_state != BREAKPOINT_HIT))
      MSP430_State(&mcu_state,FALSE,&cpucycl);
  status = MSP430_Registers(deviceRegs, (1<<PC), READ);
  //MSP430_Read_Registers(&reg,PC);
  printf("uC broke at 0x%x\n",reg);
    
  threadFinish(); 

  /*
 
  buf[0] = 56;
  buf[1] = 78;
  status = MSP430_Write_Memory(0x0200, &(buf[0]), 2);
  status = MSP430_Read_Memory(0x200,&(buf0[0]),2);
  */

}

static void
mspattach(iFid** f, char *spec)
{
  fprintf(stderr, "mspattach\n");
  *f = newifid(-1, &msperrorstrt);
  (*f)->type = devno('M');
  (*f)->qid.path = (msptab[0].qid + (*f)->type * 256) | DMDIR;
}


static void
mspwalk(iFid *f, char *name)
{
  fprintf(stderr, "mspwalk\n");
  devwalk(f, name, msptab, nelem(msptab), &msperrorstrt);
}

void
mspstat(iFid *f, Dir *d, ErrorStruct *es)
{
  fprintf(stderr, "mspstat\n");
  devstat(f, d, &msptab[f->qid.path & 255], es);

}

static void
mspopen(iFid **f, int omode, ErrorStruct *es)
{  
  fprintf(stderr, "mspopen\n");
  devopen(f, msptab, omode,0, nelem(msptab), es);
}

void
mspinit(Dev *d)
{
  volatile long status; 
 // long lvarx;
  long lvarx,cpucycl,reg;
  long mcu_state = 0;
  char memory_read[6];
  device = (union DEVICE_T*) &idbuffer[0]; // Device information structure.
   
  printf("Inside mspinit.\n");
  mspdevT.run = mspmain;
  setThrdName(mspdevT,'M');
  
  status = MSP430_Initialize(0, &lvarx);
  status = MSP430_Identify( &idbuffer[0], 64, FALSE);
  status = MSP430_Reset(VCC_RESET, FALSE, FALSE);
  printf("Device identified as: %s\n",sdeviceID[idbuffer[2]].IDstring);
 
 
  
  /*
   
  status = MSP430_Write_Memory(0xE000+8, &prog_code2[0], 7);           // even address, odd count
  if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem1 status = %ld\n",status);
   
  status = MSP430_Write_Memory(0xE000+8+7, &prog_code3[0], 4);         // odd address, even count
  if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem2 status = %ld\n",status);
     
  status = MSP430_Write_Memory(0xE000+8+7+4, &prog_code4[0], 9);       // odd address, odd count
 if (status<0) {
   printf("MSP430_Erase faile. exiting\n");
   return;
  }
  printf("MSP430_mem3 status = %ld\n",status);
   
  status = MSP430_Write_Memory(0xFFFE, &reset_vec[0], 2);              // reset vector

  MSP430_Breakpoint(0, 0xE00E);
  
  MSP430_Run(RUN_TO_BREAKPOINT,FALSE);
  printf("waiting for break ...\n");
  while(mcu_state != BREAKPOINT_HIT)
      MSP430_State(&mcu_state,FALSE,&cpucycl);
  MSP430_Read_Registers(&reg,1);
  printf("uC broke at 0x%x\n",reg);
  */

  deviceRegs[PC] = 0xE000;

  _threadready(&mspdevT);
  
  
}

void mspclone(iFid **ppc, iFid *pc) {
  devclone(ppc, pc, &msperrorstrt);
}

void mspcreate(iFid* pc, char* pch, int i, ulong ul) {
  devcreate(pc, pch, i, ul, &msperrorstrt);
}

void mspclose(iFid *pc) {
  devclose(pc, &msperrorstrt);
}

void mspremove(iFid* pc, ErrorStruct *es) {
   devremove(pc, es);
}

void mspwstat(iFid* pc, Dir* pd, ErrorStruct *es) {
   devwstat(pc, pd, es);
}


Dev mspdev = { 
  'M',
  "msp",
  &mspChan,
  &mspfidChan,
  mspinit,
  mspattach,
  mspclone,
  mspwalk,
  mspstat,
  mspopen,
  mspcreate,
  mspclose,
  mspread,
  mspwrite,
  mspremove,
  mspwstat
};


