/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

#include <stdio.h>
#include "arch/thread.h"
#include "system.h"
#include "altera_avalon_uart_regs.h"
#include "sys/alt_irq.h"

#define BUFSIZE 600
typedef unsigned char u_int8_t;

Thread consumerT;
Thread producerT;

int charcount;

Channel intChan = CHANNELN(int, 4);

int valarr[10];
FILE* fpw;

u_int8_t uartbuff[BUFSIZE];

u_int8_t state;
u_int8_t stateval;

/* 
 * The two pointers below govern reading and writing to the uart buffer. *
 * When writeptr == readptr, it means the buffer is empty                *
 * When writeptr = (readptr + 1)%BUFSIZE it means its full               *
 */
int uartbuff_write, uartbuff_read;
int overflow;

static void handle_uart_interrupts1(void* context, alt_u32 id)
{
 u_int8_t c;
 if (IORD_ALTERA_AVALON_UART_STATUS(UART1_BASE) & ALTERA_AVALON_UART_STATUS_RRDY_MSK) {
   
   c = IORD_ALTERA_AVALON_UART_RXDATA(UART1_BASE);
   state = 1 - state;
   if (!state) {
     if (((uartbuff_write+1) % BUFSIZE) == uartbuff_read) {
      overflow = 1;
      return;
     }  
     c = stateval + (c << 4); // LSN comes in first
     uartbuff[uartbuff_write] =  c;
     uartbuff_write = (uartbuff_write + 1) % BUFSIZE;  
     }
     else
       stateval = c;
   return;
 }
}

static void handle_uart_interrupts(void* context, alt_u32 id)
{
 u_int8_t c;
 int next;
 if (IORD_ALTERA_AVALON_UART_STATUS(UART1_BASE) & ALTERA_AVALON_UART_STATUS_RRDY_MSK) {
   
   c = IORD_ALTERA_AVALON_UART_RXDATA(UART1_BASE);
 

   if (((next = (uartbuff_write+1) & 0x1FF)) == uartbuff_read) {
      overflow = 1;
      return;
   }  
  
   
/*   
   if (((next = (charcount+1) & 0x1FF)) == uartbuff_read) {
      overflow = 1;
      return;
   }  
*/
   
  uartbuff[uartbuff_write] =  c;
  uartbuff_write = next ;  
//   uartbuff[charcount] = c;
 //  charcount = next;
   
   return;
 }
}

int serialsend(void *data, int len, int flags)
{
  int n;
  

  
  
   if ((n = fwrite(data, sizeof(char), len, fpw)) != len) {
     printf("error sending data out; wanted to send %d, managed %d\n", len, n);
     return n;
   }
   return n;
}

void inituart() {
 /* initialize the head and tail pointers */
 uartbuff_write = 0;
 uartbuff_read = 0;
 
 /* associate the ISR with the irq   */
 alt_irq_register( UART1_IRQ, 0, handle_uart_interrupts ); 
 
 /* enable the rcv char interrupt */
 IOWR_ALTERA_AVALON_UART_CONTROL(UART1_BASE, ALTERA_AVALON_UART_CONTROL_RRDY_MSK);
 
 charcount = 0;
 state = 0;
 stateval = 0;
 
}

int mygetchar1() {
  u_int8_t  ch;
 
 //char ch;
  /* make this SAFE!  */
  if (uartbuff_write == uartbuff_read) // empty
   return -1;
   else {
     ch = uartbuff[uartbuff_read];
     uartbuff_read = (uartbuff_read + 1) % BUFSIZE;
     charcount++;
     return (int)ch;
   }
}

int mygetchar() {
  u_int8_t  ch1;
  u_int8_t  ch2;
  int tempreadptr;
 //char ch;
  /* make this SAFE!  */
  if (uartbuff_write == uartbuff_read) // empty
   return -1;
   else 
    ch1 = uartbuff[uartbuff_read];
  tempreadptr = (uartbuff_read + 1) & 0x1FF;
  if (uartbuff_write == tempreadptr) // empty 
   return -1;
   else 
    ch2 = uartbuff[tempreadptr];
 // printf("gfc : ch1 = %d, ch2 = %d\n", ch1, ch2),
   uartbuff_read = (tempreadptr + 1) & 0x1FF;
   ch1 -= '0'; ch2 -= '0';
   return ch1 + (ch2 << 4); // LSN comes in first
  
}
     

void producer()
{

 static int val;
 
 threadBegin();
 
 for(val = 0;1; val++) {
 
  while (1) {
    if ((valarr[val] = mygetchar()) < 0)
        threadReturn(); 
      else break;
   } 
  if (chansend(&intChan, &valarr[val]) < 0)
   threadReturn();
 }
 threadFinish();
}

void consumer()
{
 static int i;
 static int chani;
 
 threadBegin();
 for(i = 0; i<10; i++) {
  if (chanrecv(&intChan, &chani) < 0)
    threadReturn();
  printf("consumer:: received on intchan %d\n", chani);
 }
 threadFinish();
   
}

int main1()
{
  printf("Hello from Nios II!\n");
  inituart();
  handlerBegin();
  consumerT.run = consumer;
  setThrdName(consumerT,'C');
  //  strcpy(srvT.name,"server");
  _threadready(&consumerT);

  producerT.run = producer;
  setThrdName(producerT,'P');
  //  strcpy(srvT.name,"server");
  _threadready(&producerT);
  
  _threadsched();
  return 0;
}
