#include <string.h>
#include <stdio.h>
#include "dat.h"
#include "fns.h"
#include "error.h"
#include "mount.h"

#define UIDOFFSET NAMEREC
#define GIDOFFSET 2*NAMEREC
#define PATHOFFSET GIDOFFSET+NAMEREC
#define VERSIONOFFSET PATHOFFSET+4
#define MODEOFFSET VERSIONOFFSET+4
#define ATIMEOFFSET MODEOFFSET+4
#define MTIMEOFFSET ATIMEOFFSET+4
#define LENGTHOFFSET MTIMEOFFSET+4
#define TYPEOFFSET LENGTHOFFSET+8
#define DEVOFFSET TYPEOFFSET+2


extern ulong kerndate;


int
devno(int c)
{
  int i;

  for(i = 0; devtab[i]; i++) {
    if(devtab[i]->dc == c)
      return i;
  }
  return -1;
}


void
devstat(iFid *f, Dir *d, const Dirtab *tab, ErrorStruct *es)
{
  static const char *nobody = "nobody";
  u_int8_t cpy[116];
  u_int32_t v32;
  u_int16_t v16;
  u_int64_t v64;

  memset(&(cpy[0]), 0, 116);

  strncpy(&(cpy[0]),tab[0].name, NAMEREC);
  
  strncpy(&(cpy[UIDOFFSET]), nobody, NAMEREC);
  strncpy(&(cpy[GIDOFFSET]), nobody, NAMEREC);
   

  v32 = tab[0].qid & (f->dev * 256); 
  if (tab[0].perm & PDIR)
    v32 |= DMDIR;
     
  memcpy(&(cpy[PATHOFFSET]),&v32,4);
  
  v32 = 0;
  memcpy(&(cpy[VERSIONOFFSET]),&v32,4);
    
  v16 = devtab[f->type]->dc;
  memcpy(&(cpy[TYPEOFFSET]),&v16,2);
  
  v16 = f->dev;
  memcpy(&(cpy[DEVOFFSET]),&v16,2);
  
  v32 = ((tab[0].perm & 7) | ((tab[0].perm & 7) << 3) 
      |((tab[0].perm & 7) << 6));
  if (tab[0].perm & PDIR)
    v32 |= DMDIR;
  memcpy(&(cpy[MODEOFFSET]),&v32,4);
  
  v32 = seconds();
  memcpy(&(cpy[ATIMEOFFSET]),&v32,4);
  
  v32 = kerndate;
  memcpy(&(cpy[MTIMEOFFSET]),&v32,4);
    
  v64 = 0;
  memcpy(&(cpy[LENGTHOFFSET]),&v64,8);

  memcpy(d,&(cpy[0]),116);

}


// this version of dirread allows offsets to be specified while reading directories. Note offsets should be multiples of DIRLEN
void
dirread(iFid *f, Dir *d, const Dirtab *tab, ushort tlen, ushort *len, int offset, ErrorStruct *es)
{
  int nofound = 0;
  
  f->dri = 0;

  for(f->dri++; f->dri < tlen; f->dri++)
    {
      /* The link field in tab actually refers to the parent.
         Hence the following line may be interpreted as: if the parent of the
	 file being considered is our dir file being read, then copy data corresp
         to it in the return data.*/
      if ((f->qid.path & 255) != tab[f->dri].link1)
	continue;
      if (nofound < offset) {
	nofound+=DIRLEN;
	continue;
      }
	
      //      devstat(c, d, &tab[c->dri]);
      devstat(f, d, &tab[f->dri], es);
      f->offset += sizeof(Dir);
      *len = sizeof(Dir);
      return;
    }
  *len = 0;
  return;
}


void	 
devclose(iFid *f, ErrorStruct *es)
{
  //   USED(f);
}

/*
 *  Walk needs to be made generic
 */

void devwalk(iFid *f, char *name, const Dirtab *tab, int len, ErrorStruct *es)
{
  int i;

  if (name[0] == '.' && name[1] == 0)
    return;
  if (name[0] == '.' && name[1] == '.' && name[2] == 0)
    {

      if (undomount(f)) {
	devtab[f->type]->walk(f, "..");
	return;
      }
      f->qid.path = (tab[f->qid.path & 255].link1 + f->type * 256) | DMDIR;
      f->qid.version =  0;

      return;
    }
  if (!(f->qid.path & DMDIR))
    error(es, "Not a directory");

  /* in inf3, we walk just one level */
  for (i = 1; i < len; i++)
    {         /* if filename as well as parent matches */
      if ((strcmp(name, tab[i].name) == 0 ) &&
	  ((f->qid.path & 255) == tab[i].link1))
	{
	  f->qid.path = tab[i].qid + f->type*256;
	  if (tab[i].perm & PDIR)
	    f->qid.path |= DMDIR;
	  f->qid.version = 0;
	  return;
	}
    }
  error(es, "Walk failed");
  
}

void
devopen(iFid **f, const Dirtab *tab, int omode, int defperm, int tabsize, ErrorStruct *es)
{
  int index = (*f)->qid.path & 255;
/*   static const int access[] = {[OREAD]  DMREAD, */
/* 			       [OWRITE] DMWRITE, */
/* 			       [ORDWR]  DMREAD | DMWRITE, */
/* 			       [OEXEC]  DMEXEC}; */
  int t; //= access[omode & 3];

  switch (omode & 3) {
  case OREAD:
    t = DMREAD;
    break;
  case OWRITE:
    t = DMWRITE;
    break;
  case ORDWR:
    t = DMREAD | DMWRITE;
    break;
  case OEXEC:
    t = DMEXEC;
    break;
  }
  
  if (index > (tabsize-1)) {
    if ((t & defperm)!= t) 
      error(es, Eperm);
  }
  else
  if((t&tab[index].perm) != t)
    error(es, Eperm);
  
  (*f)->offset = 0;
  (*f)->dri    = 0;
  if(((*f)->qid.path&DMDIR) && omode!=OREAD)
    error(es, Eperm);
  (*f)->mode = omode & (OEXEC);
  (*f)->flag |= COPEN;

}


void
devclone(iFid **nf, iFid *f, ErrorStruct *es)
{

  if(f->flag & COPEN)
    error(es, "clone of open file");
    //    error(es, "clone of open file  %C\n", devtab[c->type]->dc);

  *nf = newifid(0, es);

  (*nf)->type = f->type;
  (*nf)->dev = f->dev;
  (*nf)->mode = f->mode;
  (*nf)->qid = f->qid;
  (*nf)->offset = f->offset;

  // if this is a mount point then the from must definitely be the root
  //  and hence is local. So there is no way that is going to block
  //  if (c->from)
  //  cclone(&((*nc)->from), c->from, 0); 
}

void
devinit(void)
{
  printf("Inside devinit.\n");
}

void
devcreate(iFid* f, char* n, int i, ulong l, ErrorStruct *es)
{
  //  USED(c); USED(n); USED(i); USED(l);
  error(es, Eperm);
}


void
devremove(iFid *f , ErrorStruct *es)
{
  //  USED(c);

  //  strncpy(es->errstr, Eperm, ERRMAX);
  //  longjmp(es->estack, 0);
}

void
devwstat(iFid* f, Dir* d, ErrorStruct *es)
{
  //  USED(c); USED(d);
  //  error(es, Eperm);
}


