/*
 *  template for making a new device
 */

/*
 * Notes -- support single large multidirectory
 *          table
 *       -- use device info to find index start (dump pointer arithmetic)
 */

/*
 *  Qids must be in range 0-255.  chan qid.path computed as 
 *  c->type *  256 + Qid 
 */

#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <string.h>
#include "error.h"
#include "dat.h"
#include "fns.h"
#include "styx.h"
#include "arch/thread.h"

/*
 * Device specfic stuff
 */


extern Dev testdev;

enum{
  Qdir,
    Qdata,
    Qdata2,
    Qdir2,
    Qdata3,
    Qdata4
};

static const Dirtab
testtab[]={
  {".",    Qdir,   Qdir,  PDIR | 05},
  {"data", Qdata,  Qdir,  06},
  {"a",    Qdata2, Qdir,  04},
  {"b",    Qdir2,  Qdir,  PDIR | 05},
  {"c",    Qdata3, Qdir2, 04},
  {"d",    Qdata4, Qdir2, 04}
};


#define BUFSIZE 256
static char data[BUFSIZE];

/* pipe for outgoing packets */
//extern Callpipe outpipe;
extern Channel outChan;

Channel testChan = CHANNELN(Fcall *, 4);
Channel testfidChan = CHANNELN(iFid *, 4);


ErrorStruct testerrorstrt;

Thread testT;

static void
testread(iFid *c, void *buf, ushort n, vlong offset, ushort *len);

static void
testwrite(iFid *c, void *buf, ushort n, vlong offset, ushort *len);

static void
testwalk(iFid *c, char *name);


/*
  This is the control loop for the test device.
  It receives calls through the call pipe and dispatches them to the corresponding function.
*/

void testmain() {
  //  Fid *f;
  static Fcall *rx;
  static Fcall *tx;
  static iFid *f;
  char cmd;
  void *buf;
  u_int16_t count;
  u_int64_t offset;
  u_int8_t index;
  char *name;
  
  printf("testmain: rcving on cpipe %p\n",&testChan);

  threadBegin();  
  while(1) {
    // receive call from the call pipe
    if (chanrecv(&testChan, &rx) < 0) 
      
      threadReturn();
  

    printf("testthread: recvd 1  request\n");
    if (chanrecv(&testfidChan, &f) < 0)
      threadReturn();
    if (chanrecv(&testChan, &tx) < 0)
      threadReturn();

  if (!(setjmp(testerrorstrt.estack))) 
   switch(rx->type) {
	
  case Tread:
	count = rx->ns.un.Twrf.count;
	offset = rx->ns.un.Twrf.offset;
	testread(f, tx->ns.un.Rwrf.rdata + 1, count, offset, &(tx->ns.un.Rwrf.rcount));    
	break;
	
  case Twrite:
	count = rx->ns.un.Twrf.count;
	offset = rx->ns.un.Twrf.offset;
	buf = rx->ns.un.Twrf.data;
	testwrite(f, buf, count, offset, &(tx->ns.un.Rwrf.rcount));
	break;
	
  case Twalk:
	name = rx->ns.un.Twcf.name;
	testwalk(f, name);
	tx->ns.un.qid = f->qid;
	break;
	
      default:
	tx->type = Rerror;
	strcpy(tx->ename, "Invalid operation on testdev");
      }
      else
	{
	  tx->type = Rerror;    
	  strncpy(tx->ename, testerrorstrt.errstr, ERRREC);
	}

    if (chansend(&outChan, &tx) < 0)
      threadReturn();    


    // call the corresponding function

    // send the response out of the output pipe

    // return the buffer
  }
  threadFinish();

}

static void
testread(iFid *f, void *buf, ushort n, vlong offset, ushort *len)
{
  char *tmp;
  int  slen;

  int status;

  fprintf(stderr, "testread\n");
  
  if (f->qid.path & DMDIR)
    {
      dirread(f, (Dir *) buf, testtab, nelem(testtab), len, offset, &testerrorstrt);
      return;
    }

  /* local file */

  switch(f->qid.path & 255) {
  case Qdata:
    {
      int end = (offset + n >= BUFSIZE) ? (BUFSIZE - 1) : offset + n;
      if (offset >= BUFSIZE)
	{
	  *len = 0;
	  return;
	}
      else
	memcpy(buf, &data[offset], end - offset);
      *len = end - offset;
      return;
    }
  case Qdata2:
    tmp = "testa\n";
    slen = strlen(tmp) + 1;
    if (offset > slen)
      {
	*len = 0;
	return;
      }
    if ((n + offset) > slen)
      n = slen - offset;
    memcpy(buf, &tmp[offset], n);
    *len = n;


    return;
    break;
  case Qdata3:
    tmp = "testc\n";
    break;
  case Qdata4:
    tmp = "testd\n";
    break;
  default:
    *len = 0;
    return;
  }
  slen = strlen(tmp) + 1;
  if (offset > slen)
    {
      *len = 0;
      return;
    }
  if ((n + offset) > slen)
    n = slen - offset;
  memcpy(buf, &tmp[offset], n);
  *len = n;
  return;
}

static void
testwrite(iFid *f, void *buf, ushort n, vlong offset, ushort *len)
{
  fprintf(stderr, "testwrite\n");
  if ((f->qid.path & 255) == Qdata)
    {
      int end   = (offset + n >= BUFSIZE) ? (BUFSIZE - 1) : offset + n;
      if (offset >= BUFSIZE)
	{
	  *len = 0;
	  return;
	}
      else
	memcpy(&data[offset], buf, end - offset);
      *len = end - offset;

      return;
    }
  else
    error(&testerrorstrt, Eperm);
}

static void
testattach(iFid** f, char *spec)
{
  fprintf(stderr, "testattach\n");
  *f = newifid(-1, &testerrorstrt);
  (*f)->type = devno('T');
  (*f)->qid.path = (testtab[0].qid + (*f)->type * 256) | DMDIR;
}


static void
testwalk(iFid *f, char *name)
{
  fprintf(stderr, "testwalk\n");
  devwalk(f, name, testtab, nelem(testtab), &testerrorstrt);
}

void
teststat(iFid *f, Dir *d, ErrorStruct *es)
{
  fprintf(stderr, "teststat\n");
  devstat(f, d, &testtab[f->qid.path & 255], es);

  switch (d->qid.path & 255)
    {
    case Qdir : 
    case Qdir2:
      d->length = 2; 
      break;
    case Qdata2 :
    case Qdata3 :
    case Qdata4 :
      d->length = 8; 
      break;
    }
  if (f->flag&CMSG)
    d->mode != DMMOUNT;

}

static void
testopen(iFid **f, int omode, ErrorStruct *es)
{  
  fprintf(stderr, "testopen\n");
  devopen(f, testtab, omode,0, nelem(testtab), es);
}

void
testinit(Dev *d)
{
  printf("Inside testinit.\n");
  testT.run = testmain;
  setThrdName(testT,'E');
  _threadready(&testT);
  
}

void testclone(iFid **ppc, iFid *pc) {
  devclone(ppc, pc, &testerrorstrt);
}

void testcreate(iFid* pc, char* pch, int i, ulong ul) {
  devcreate(pc, pch, i, ul, &testerrorstrt);
}

void testclose(iFid *pc) {
  devclose(pc, &testerrorstrt);
}

void testremove(iFid* pc, ErrorStruct *es) {
   devremove(pc, es);
}

void testwstat(iFid* pc, Dir* pd, ErrorStruct *es) {
   devwstat(pc, pd, es);
}


Dev testdev = {	
  'T',
  "test",
  &testChan,
  &testfidChan,
  testinit,
  testattach,
  testclone,
  testwalk,
  teststat,
  testopen,
  testcreate,
  testclose,
  testread,
  testwrite,
  testremove,
  testwstat
};


