#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include "dat.h"
#include "fns.h"
#include "error.h"
#include "arch/thread.h"

//#define MAXCALLS 10
#define MAXCALLS 3
//#define stderr stdout


/*
 * beginning of server code
 */

int	rversion(Fcall*, Fcall*);
int	rattach(Fcall*, Fcall* );
int	rflush(Fcall*, Fcall* );
int	rclone(Fcall*, Fcall* );
int	rwalk(Fcall*, Fcall* );
int	ropen(Fcall*, Fcall* );
int	rcreate(Fcall*, Fcall* );
int	rreadwrite(Fcall*, Fcall* );
int	rclunk(Fcall*, Fcall* );
int	rstat(Fcall*, Fcall* );
int	rwstat(Fcall*, Fcall* );
int	rremove(Fcall*, Fcall* );

void
singleputfcall(Fcall *tx);

ulong messagesize = 128;
uchar* rxbuf;
uchar* txbuf;
void *databuf;
int connected;
iFid *root;

/* Error structure for the serve thread */ 
ErrorStruct serveerrstrt; 
ErrorStruct removeerrstrt; 

/* channel for incoming packets  */
extern Channel inChan;

/* channel for sending out packets */
extern Channel outChan;

/* channel to return fcall buffers   */
extern Channel rtnbufferChan;

extern iFid *fidtab;

/* to store incoming packets */
Fcall fcrar[MAXCALLS];

/* to store outgoing packets */
Fcall fctar[MAXCALLS];

/* stores which indices are free and available */
int fcavail[MAXCALLS];

/* shared flags between serve() and routines to indicate completion */
char fcstatus[MAXCALLS];



/* return an index that corresponds to an available fcall location */
int fcalloc()
{
  int i;
  for(i=0;i<MAXCALLS;i++)
    if (fcavail[i] == 0) {
      fcavail[i] = 1;
      return i;
    }
  return -1;
}

/* return an fcall index into available pool */
void fcfree(int n)
{
  fcavail[n] = 0;
}

extern Dev *rootdev;

void *bufrtnthread(void *ignore) {
  Fcall *tx;
  int index;

  while(1) {
    if (chanrecv(&rtnbufferChan, &tx) < 0)
      threadReturn();

    index = (tx - &(fctar[0]))/sizeof(Fcall);
    fcfree(index);
  }
  return NULL;
}
    
void
serve()
{
  static Fcall *rx, *tx;
  static Fcall rxtemp;

  /* following not static as not used across blocks */
  int i;  
  int rxi,txi;

  threadBegin();

  devtab[0]->attach(&root, "");


  for(;;){

    if (chanrecv(&inChan, &rxtemp) < 0)
      threadReturn();

    //      printf("serve: received packet, type = %d\n",rxtemp.type);
    i = fcalloc();
    memcpy(&(fcrar[i]), &rxtemp, sizeof(Fcall));
    
    rx = &(fcrar[i]);
    tx = &(fctar[i]);    
    memset(tx, 0, sizeof(Fcall));
    tx->type = rx->type+1;
    tx->tag = rx->tag;
    tx->ns.fid = rx->ns.fid;
    
    if (!(setjmp(serveerrstrt.estack))) {      
      switch(rx->type){
      case Tflush:
	break;
      case Tattach:
	if (rattach(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Tclone:
	if (rclone(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Twalk:
	if (rwalk(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Tstat:
	if (rstat(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Twstat:
	if (rwstat(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Topen:
	if (ropen(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Tcreate:
	if (rcreate(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Tread:
	rxi = rx->ns.un.Twrf.count;
	txi = sizeof(tx->ns.un.Rwrf.rdata);
	if (rxi >= txi)
	  rx->ns.un.Twrf.count = sizeof(tx->ns.un.Rwrf.rdata) - 1;
	if (rreadwrite(rx, tx) < 0)       // handled by device
	  error(&serveerrstrt, Emessage);
	break;
      case Twrite:
	if (rreadwrite(rx, tx) < 0)        // handled by device
	  error(&serveerrstrt, Emessage);
	break;
      case Tclunk:
	if (rclunk(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Tremove:
	if (rremove(rx, tx) < 0)
	  error(&serveerrstrt, Emessage);
	break;
      case Tnop:  //take care of this
	printf("rnop\n");
	if (chansend(&outChan, &tx) < 0)
	  threadReturn();
	break;
      default:
	error(&serveerrstrt, Emessage);
	break;
      }
    }
    else {
      tx->type = Rerror;
      printf("rerror from serve: %s\n",serveerrstrt.errstr);
      strcpy(tx->ename, serveerrstrt.errstr);
      if (chansend(&outChan, &tx) < 0)
	       threadReturn();
      }
  }
  threadFinish();
}


int
rattach(Fcall *rx, Fcall *tx)
{
  iFid *fid;

  //  fprintf(stderr, "rattach\n");

  if (rx->ns.un.Tattf.aname == nil)
    strcpy(rx->ns.un.Tattf.aname, "");

  /*
  if (waserror())
    {
      freefid(fid);
      nexterror();
    }
  */

  printf("rattach:fid = %d\n",rx->ns.fid);
  ifidclone(&fid , root, &serveerrstrt);
  fid->fidno = rx->ns.fid;

  /* need to throw an error of fid->chan = nil */
  
  /* should check users etc. */

  tx->ns.un.qid = fid->qid;

  if (chansend(&outChan, &tx) == LASTALT)
    return -1;
  else return 1;
      
  //  printf("serve: sent pkt out on outpipe\n");
  //  poperror();
}

/*
 * rwalk should be modified to do this safely --
 *  in particular, . has to be handled to keep
 *  within exported file system
 */

int rwalk(Fcall *rx, Fcall *tx)
{
 
  iFid *fid;
  fprintf(stderr, "rwalk\n");
  fid = lookupfid(rx->ns.fid);
  return walk(&fid, rx, tx, devtab[fid->type]->cpipe, devtab[fid->type]->cfidpipe, &serveerrstrt);
}

int
ropen(Fcall *rx, Fcall *tx)
{
  iFid *fid;
  fid = lookupfid(rx->ns.fid);
  fprintf(stderr, "ropen\n");
  fidopen(&fid, rx->ns.un.omode, &serveerrstrt);
  tx->ns.un.qid = fid->qid;
  if (chansend(&outChan, &tx) == LASTALT)
    return -1;
  else return 0;

  //  printf("serve: sent pkt out on outpipe\n");
}

int
rcreate(Fcall *rx, Fcall *tx)
{
 
  iFid *fid = lookupfid(rx->ns.fid);
  fprintf(stderr, "rcreate\n");
  devtab[fid->type]->create(fid, rx->ns.un.Twcf.name, rx->ns.un.Twcf.Tcf.cmode, rx->ns.un.Twcf.Tcf.perm);
  tx->ns.un.qid = fid->qid;
  if (chansend(&outChan, &tx) == LASTALT)
    return -1;
  else return 1;
      

}

/*
Call to the device's read procedure is made through a callpipe and is no longer a function call. So fsexport passes the call over the callpipe and forgets about it. SO where is this callpipe obtained from?
*** from the device table
*** There is a 'process' that waits on this callpipe for each device and proceses the call

The idea is that this callpipe abstraction can be implemented in many ways:
* on a workstation, using threads and semaphores
* on a msp430, using lightweight threads and transactional channels

It just needs the ability to have process/thread wait on it for data or instructions (without having to busy wait)

Question: do we pass the buffer around or just the arguments?

 */

int
rreadwrite(Fcall *rx, Fcall *tx)
{
  iFid *fid;
 

  fprintf(stderr, "rread/write\n");

  fid = lookupfid(rx->ns.fid);
  if (channbsend(devtab[fid->type]->cpipe, &rx) == LASTALT)
    return -1;
  if (channbsend(devtab[fid->type]->cfidpipe, &fid) == LASTALT)
    return -1;
  if (channbsend(devtab[fid->type]->cpipe, &tx) == LASTALT)
    return -1;

  return 1;
}

int
rclunk(Fcall *rx, Fcall *tx)
{
  iFid *fid = lookupfid(rx->ns.fid);
  fprintf(stderr, "rclunk: fid %d\n", rx->ns.fid);
  freeifid(fid);
  tx->ns.fid = rx->ns.fid;

  if (channbsend(&outChan, &tx) == LASTALT)
    return -1;
  else return 1;
  //  printf("serve: sent pkt out on outpipe\n");
}

int
rstat(Fcall *rx, Fcall *tx)
{

  iFid *fid = lookupfid(rx->ns.fid);
  fprintf(stderr, "rstat\n");
  devtab[fid->type]->stat(fid, (Dir *) &tx->ns.un.dir, &serveerrstrt);

  if (channbsend(&outChan, &tx) == LASTALT)
    return -1;
  else return 1;

  //  printf("serve: sent pkt out on outpipe\n");
}

int
rwstat(Fcall *rx, Fcall *tx)
{
 
  iFid *fid = lookupfid(rx->ns.fid);
  fprintf(stderr, "rwstat\n");
  devtab[fid->type]->wstat(fid, (Dir *) &tx->ns.un.dir, &serveerrstrt);
  if (channbsend(&outChan, &tx) == LASTALT)
    return -1;
  else
    return 1;

}

int
rclone(Fcall *rx, Fcall *tx)
{

  iFid *fid;
  iFid *nfid;

  fprintf(stderr, "rclone: %d to %d\n", rx->ns.fid, rx->ns.un.newfid);

  /*
  if (waserror())
    {
      freefid(nfid);
      nexterror();
    }
  */

  fid = lookupfid(rx->ns.fid);;

  ifidclone(&nfid, fid, &serveerrstrt);
  nfid->fidno = rx->ns.un.newfid;
  if (channbsend(&outChan, &tx) == LASTALT)
    return -1;
  else
    return 1;

  //  printf("serve: sent pkt out on outpipe\n");
  //  poperror(); 

}

/* remove must clunk fid even if remove fails */

int
rremove(Fcall *rx, Fcall *tx)
{
  
  iFid *fid = lookupfid(rx->ns.fid);

  fprintf(stderr, "rremove\n");

  /*
  if (waserror())
    {
      freefid(fid);
      nexterror();
    }
  */

  /*
  if (!(setjmp(removeerrstrt.estack))) 
    devtab[fid->chan->type]->remove(fid->chan, &removeerrstrt);
  else {
    freefid(fid);
    error(serveerrstrt, removeerrstrt.errstr);
  } 
  */   
  freeifid(fid);
  if (channbsend(&outChan, &tx) == LASTALT)
    return -1;
  else
    return 1;

  //  poperror();
}

