#include <string.h>
#include "dat.h"
#include "fns.h"
#include "mount.h"

#define MAXMOUNTS 16
#define ROOTDEV 'R'

typedef struct Mount Mount;
struct Mount {
  iFid* from;
  iFid* to;
};

/* typedef struct MountList MountList; */

/* struct MountList { */
/*   Chan* from; */
/*   Chan* to; */
/*   MountList* next; */
/* }; */

/* MountList* mlist = nil; */

Mount mounts[MAXMOUNTS];
int nextmountidx = 0;

int iFideq(iFid* a, iFid* b) {
  if (a && b) {
    if (a->type == b->type &&
	a->dev  == b->dev  &&
	a->qid.path == b->qid.path) {
      return 1;
    } else {
      return 0;
    }
    return 0;
  }
   return 0;
}

iFid* mountlookup(iFid* from, iFid* to) {
  int i;
  if (from) {
    for(i = 0; i < MAXMOUNTS; i++) {
      if (iFideq(mounts[i].from, from))
	return mounts[i].to;
    }
    return nil;
  } else if (to) {
    for(i = 0; i < MAXMOUNTS; i++) {
      if (iFideq(mounts[i].to, to))
	return mounts[i].from;
    }
    return nil;
  } 

  return nil;

}


void setnextmountidx() {
  int i;
  for (i = (nextmountidx + 1) % MAXMOUNTS; i != nextmountidx; i = (i + 1) % MAXMOUNTS) {
    if (mounts[i].from == nil && mounts[i].to == nil) {
      nextmountidx = i;
      return;
    }
  }
}

// creates an association between 'from' and 't'o
// 'from' is now a directory that contains the files
// in 'to', which should be pointed at the root of a device fs.

int mount(iFid* from, iFid* to) {
  if (devno(ROOTDEV) != from->dev) {
    return 0;
  }
  if (nextmountidx != -1) {
    mounts[nextmountidx].from = from;
    mounts[nextmountidx].to = to;
    setnextmountidx();
    return 1;
  } else {
    return 0;
  }
}

// unmounts the fid previously mounted to 'from'
int unmount(iFid* from) {
  int i;
  for (i = 0; i < MAXMOUNTS; i++) {
    if (iFideq(mounts[i].from, from)) {
      mounts[i].from = nil;
      mounts[i].to = nil;
      if (nextmountidx == -1)
	setnextmountidx();
    }
  }

  return 1;
	
}

int domount(iFid* from) {
  iFid* to;
  iFid* prev;
  iFid* next;
  int fidno;

  if ((to = mountlookup(from, nil))) {
    fidno = from->fidno;   // the fid needs to be preserved across the operation...
    prev = from->prev;
    next = from->next;
    memcpy(from, to, sizeof(iFid));
    from->fidno = fidno;   // restore the fid
    from->prev = prev;
    from->next = next;
    return 1;
  }
  return 0;
}

int undomount(iFid* to) {
  iFid* from;
  int fidno;
  iFid* prev;
  iFid* next;

  if ((from = mountlookup(nil, to))) {
    fidno = to->fidno;  // the fid needs to be preserved across the operation...
    prev = to->prev;
    next = to->next;
    memcpy(to, from, sizeof(iFid));
    to->fidno = fidno;  // restore the fid
    to->prev = prev;
    to->next = next;
    return 1;
  }
  return 0;

}

