#ifndef STYX_H
#define STYX_H

//#include <stddef.h>

typedef unsigned char u_int8_t;
typedef short int16_t;
typedef unsigned short u_int16_t;
typedef int int32_t;
typedef unsigned int u_int32_t;
typedef long long int64_t;
typedef unsigned long long u_int64_t;
typedef u_int16_t ushort;
typedef u_int32_t ulong;


#define	DIRLEN		116

typedef struct Fcall Fcall;
typedef struct Qid9p Qid9p;
typedef struct Dir Dir;


#define MAXWELEM 12
//#define ERRREC  64
#define NAMEREC 28

#define	OREAD	0	/* open for read */
#define	OWRITE	1	/* write */
#define	ORDWR	2	/* read and write */
#define	OEXEC	3	/* execute, == read but check execute permission */



/* bits in Qid.type */

#define QTDIR		0x80		/* type bit for directories */
#define QTAPPEND	0x40		/* type bit for append only files */
#define QTEXCL		0x20		/* type bit for exclusive use files */
#define QTMOUNT		0x10		/* type bit for mounted channel */
#define QTAUTH		0x08		/* type bit for authentication file */
#define QTFILE		0x00		/* plain file */

/* bits in Dir.mode */

#define DMDIR		0x80000000	/* mode bit for directories */
#define DMAPPEND	0x40000000	/* mode bit for append only files */
#define DMEXCL		0x20000000	/* mode bit for exclusive use files */
#define DMMOUNT		0x10000000	/* mode bit for mounted channel */
#define DMAUTH		0x08000000	/* mode bit for authentication file */
#define DMREAD		0x4		/* mode bit for read permission */
#define DMWRITE		0x2		/* mode bit for write permission */
#define DMEXEC		0x1		/* mode bit for execute permission */

enum {
        Tnop,           //  0
        Rnop,           //  1
        Terror,         //  2, illegal
        Rerror,         //  3
        Tflush,         //  4
        Rflush,         //  5
        Tclone,         //  6
        Rclone,         //  7
        Twalk,          //  8
        Rwalk,          //  9
        Topen,          // 10
        Ropen,          // 11
        Tcreate,        // 12
        Rcreate,        // 13
        Tread,          // 14
        Rread,          // 15
        Twrite,         // 16
        Rwrite,         // 17
        Tclunk,         // 18
        Rclunk,         // 19
        Tremove,        // 20
        Rremove,        // 21
        Tstat,          // 22
        Rstat,          // 23
        Twstat,         // 24
        Rwstat,         // 25
        Tsession,       // 26 -- unimplemented
        Rsession,       // 27 -- unimplemented
        Tattach,        // 28
        Rattach        // 29
};

struct Qid9p {
  u_int32_t path;
  u_int32_t version;
};

struct Dir {
  char      name[NAMEREC]; // file name
  char      uid[NAMEREC];  // user
  char      gid[NAMEREC];  // group
  Qid9p     qid   __attribute__((packed));    
  u_int32_t mode;     // permissions
  u_int32_t atime;    // access time
  u_int32_t mtime;    // modification time
  u_int64_t length;   
  u_int16_t type;     // kernel use
  u_int16_t dev;      // kernel use
};


typedef struct Tatt_struct {                // Tattach
	  char uid[NAMEREC];
	  char aname[NAMEREC];
	} Tatt;

typedef struct Tcr_struct{              // Tcreate
	    u_int32_t perm;
	    u_int8_t cmode;
	  } Tcr;

typedef struct Twc_struct{
	  char name[NAMEREC];   // Twalk, Tcreate
          Tcr Tcf;
	  } Twc;

typedef struct Twr_struct{                // Tread, Twrite
	  u_int64_t offset __attribute__((packed));
	  u_int16_t count;
	  u_int8_t  pad;
	  u_int8_t  data[111];
	} Twr;  

typedef	struct Rwr_struct{                // Rread, Rwrite
	  u_int16_t rcount;     
	  u_int8_t  rdata[120]; // pad intentional omitted !
	} Rwr;

struct Fcall {
  u_int8_t  len;
  u_int8_t  type;
  u_int16_t tag;
  union {
    u_int8_t buf[124];
    char ename[64];         // Rerror
    u_int16_t oldtag;           // Tflush
    struct {
      u_int16_t fid;
      union {
        Tatt Tattf;
	u_int8_t omode;
	Qid9p qid; // Rattach, Rwalk, Ropen, Rcreate;
	u_int16_t newfid;       // Tclone
        Twc Twcf;
        Twr Twrf;
	Rwr Rwrf;
	Dir dir;                // Rstat Twstat
      } __attribute__((packed)) un;
    } ns;
  };
};

#endif
